# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module baseclasses
##   Classes de base utilises par pycc
## ----------------------------------------------------------------------

class PyccError(Exception):
  """ Exceptions de pycc.
  """
  def __init__(self, msg=''):
    self.message = msg
  
  def __str__(self):
    return self.message
  
class Post(object):
  """ Structure d'un post sur la tribune.
  """
  def __init__(self, id='', time='', login='', info='', message='', msg_tags={}):
    self.id = int(id)
    self.time = time
    self.info = info
    self.login = login
    self.message = message
    self.num = 0
    self.tags = msg_tags
    self.original_text = ''
    self.status = []
    self.next = None
    self.prev = None
  
  def clone(self):
    import copy
    npost = copy.copy(self)
    npost.tags = self.tags.copy()
    return npost
  
  def __str__(self):
    return repr(u"Post %s (%s)" % (self.id, self.dispTimeRef()))
  
  def __cmp__(self, other):
    return self.id - other.id
  
  def set(self, name, value):
    if name == 'id':
      self.id = value
    elif name == 'time':
      self.time = value
    elif name == 'info':
      self.info = value
    elif name == 'login':
      self.login = value
    elif name == 'message':
      self.message = value
    elif name == 'num':
      self.num = value
  
  def disp(self, name):
    if name == 'id':
      return str(self.id)
    elif name == 'time':
      return self.time[8:10] + u":" + self.time[10:12] + u":" + self.time[12:14]
    elif name == 'date':
      return self.time[6:8] + u"/" + self.time[4:6] + u"/" + self.time[:4]
    elif name == 'info':
      if self.info == "":
        return ur"[vide]"
      else:
        return self.info[:10]
    elif name == 'ua':
      return self.info
    elif name == 'login':
      if self.login == u'Anonyme':
        return u""
      else:
        return self.login
    else:
      if hasattr(self, name):
        return eval("self." + name)
      else:
        return u""
  
  def compare_time(self, timeref):
    if self.time[8:10] > timeref[3:5]:
      return 1
    if self.time[8:10] == timeref[3:5]:
      if self.time[10:12] > timeref[5:7]:
        return 1
      if self.time[10:12] == timeref[5:7]:
        if timeref[7:9] == '--':
          return 0
        if self.time[12:14] > timeref[7:9]:
          return 1
        if self.time[12:14] == timeref[7:9]:
          if timeref[9] == '0':
            return 0
          if self.num > int(timeref[9]):
            return 1
          if self.num == int(timeref[9]) or (self.num == 0 and timeref[9] == '1'):
            return 0
    return -1
  
  def same_time(self, apost):
    return self.time[8:14] == apost.time[8:14]
  
  def getAllRefs(self):
    """Renvoie la liste des rfrences (format interne) susceptibles de pointer sur ce post"""
    if self.num == 0: other = '1'
    else: other = '0'
    return [self.time[8:14]+str(self.num), self.time[8:14]+other, self.time[8:12]+"--0"]
  
  def dispTimeRef(self):
    """Renvoie l'horloge-rfrence au post"""
    if self.num == 0: dnum = ur''
    elif self.num == 1: dnum = ur''
    elif self.num == 2: dnum = ur''
    elif self.num == 3: dnum = ur''
    else: dnum = u":%s" % self.num
    return self.disp('time') + dnum
  