# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module cointklib.dialogs
##   Ce module contient des dfinitions de botes de dialogue
##   gnriques utilises par l'interface de pycc
## ----------------------------------------------------------------------

import sys
sys.path.append('..')

from basicwidgets import *
#from widgets import OptionsEditor
from constants import *

class CoinTopLevel(Toplevel):
  
  def __init__(self, parent, title=None, buttons=None):
    Toplevel.__init__(self, parent)
    if title:
      self.title(title)
    self.parent = parent
    self.initial_focus = ''
    if buttons:
      body = Frame(self, border=2, relief=RAISED)
      self.initial_focus = self.body(body)
      body.pack(fill=BOTH, expand=1)
      self.buttonbox(buttons)
    self.protocol("WM_DELETE_WINDOW", self.cancel)
  
  def body(self, master):
    pass # abstract
  
  def buttonbox(self, buttons):
    box = Frame(self)
    for but in buttons:
      w = Button(box, text=but, width=10, command=self.get_callback(but))
      w.pack(side=LEFT, padx=5)
      if but == BUT_OK:
        w.config(default=ACTIVE)
    self.bind("<Return>", self.ok)
    self.bind("<Escape>", self.cancel)
    box.pack(pady=5)

  def get_callback(self, button):
    if button == BUT_OK:
      return self.ok
    elif button == BUT_CANCEL or button == BUT_CLOSE:
      return self.cancel
    elif button == BUT_APPLY:
      return self.apply
    elif button == BUT_SAVE:
      return self.save
    elif button == BUT_NEW:
      return self.new
    elif button == BUT_DEL:
      return self.delete
    else:
      return self.bind_button(button)
  
  def bind_button(self, button):
    # Mthode abstraite, par dfaut retourne une fonction nulle
    return lambda w=None, e=None: None
  
  def ok(self, event=None):
    self.withdraw()
    self.update_idletasks()
    self.apply()
    self.parent.canceled = 0
    self.cancel()
  
  def cancel(self, event=None):
    self.destroy()
  
  def apply(self, event=None):
    pass # abstract
  
  def new(self, event=None):
    pass # abstract
  
  def delete(self, event=None):
    pass # abstract
  
  def save(self, event=None):
    pass # abstract


class CoinDialog(CoinTopLevel):

  def __init__(self, parent, title=None, buttons=[ BUT_OK, BUT_CANCEL ], modal=0):
    CoinTopLevel.__init__(self, parent, title, buttons)
    self.transient(parent)
    self.result = None
    self.callbacks = {}
    if modal: self.grab_set()
    if not self.initial_focus:
      self.initial_focus = self
    if self.parent is not None:
      self.geometry("+%d+%d" % (parent.winfo_rootx()+50, parent.winfo_rooty()+50))
    self.initial_focus.focus_set()
    self.wait_window(self)
  
  def ok(self, event=None):
    if not self.validate():
      self.initial_focus.focus_set()
      return
    self.withdraw()
    self.update_idletasks()
    self.apply()
    self.parent.canceled = 0
    self.cancel()
  
  def cancel(self, event=None):
    if self.parent is not None:
      self.parent.focus_set()
    self.destroy()
  
  def validate(self):
    return 1 # abstract
  

class SelectDialog(CoinDialog):

  def __init__(self, parent, items, title=None, text='', **kwargs):
    self.items = items
    self.text = text
    self.kwargs = kwargs
    CoinDialog.__init__(self, parent, title)
  
  def body(self, master):
    Label(master, text=self.text).pack(side=TOP, padx=6)
    self.list = SortedListBox(master, self.items, **self.kwargs)
    self.list.pack(side=TOP, padx=6, pady=6, fill=BOTH, expand=1)
  
  def apply(self):
    self.parent.selected = self.list.selected
  

class OptionsDialog(CoinDialog):
  
  def __init__(self, title, parent, object, extra_arg):
    self.object = object
    self.extra_arg = extra_arg
    CoinDialog.__init__(self, parent, title, [BUT_OK, BUT_CANCEL], modal=1)
  
  def body(self, master):
    from widgets import OptionsEditor
    self.w_options = OptionsEditor(master, self.object, extra_arg=self.extra_arg)
    self.w_options.pack()
  
  def apply(self):
    self.w_options.apply()
  



"""
# ---------------- Test program -----------------------
class Essai(CoinDialog):

  def body(self, master):
    self.l = OrderedListBoxWithButtons(master, ['bonjour', 'allo', 'coucou'], height=4, width=15)
    self.l.pack()
    self.l.bind_new(lambda w=self: w.e.get())
    self.l.bind_delete(self.kikoo)
    self.e = Entry(master)
    self.e.pack()
    return self.e
  
  def validate(self):
    print "--- Dans Validate"
    if self.e.get() == "ok":
      return 1
    return 0
  
  def kikoo(self):
    print "Kikoooooo!!"
    if self.e.get() == "del":
      return 1
    return 0
  
  def new(self):
    print "--- Dans New"
    tmplist = [ 'Le Premier', 'Le Second', 'Le Troisieme', 'Le Quatrieme' ]
    self.selected = None
    SelectDialog(self, tmplist, "Coucou", "Slectionnez dans la liste", height=5, width=15)
    self.e.delete(0, END)
    self.e.insert(END, tmplist[int(self.selected)])
    self.l.addItem()

def openEssai():
  global root
  Essai(root, "Essai", [BUT_NEW, "Autre", BUT_OK, BUT_CLOSE])

if __name__ == "__main__":

  root = Tk()
  root.title("pyCoinCoin")
  root.selected = None
  fr = Frame(root)
  myimg = PhotoImage(file="pyccwiz.gif")
  Label(fr, image=myimg, border=2, relief=GROOVE).pack(side=LEFT, padx=2, pady=2)
  Button(fr, text="CoinDialog", command=openEssai).pack()
  fr.pack()
  root.mainloop()
  print root.selected
"""
