# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module forums
##   Dfinition du module de bouchot Forums :
##   monitoring des forums de linuxfr
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

import xml.dom.minidom

from constants import *
from modules.module import *
from coinlib.persistants import OptionSet, Option
from coinlib.utils import *
from coinlib.parsers import *

def build_module(parent, name):
  return Forums(parent, name)

class ForumsOptions(OptionSet):
  """ Options du module Forums
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "Forums"),
      'short_name':   Option("Nom court (pour la diode)", "F"),
      'backend':      Option("URL du fichier de backend", "/forums/"),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'encoding':     Option("Encoding du backend", "iso-8859-15"),
      'check_delay':  Option("Nombre de secondes entre 2 checks du backend", 300, TYPE_INT),
      'max_history':  Option("Nombre maximum d'lments  conserver", 50, TYPE_INT),
      'read_at_exp':  Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'tags_encoded': Option("Tags encods", 0, TYPE_BOOL),
      'display_img':  Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'short_name', 'backend', 'check_delay', 'max_history', 'read_at_exp', 'docked' ],
          "Parsing": [ 'tags_encoded', 'display_img', 'encoding' ]
        } )
    }


class ForumItem(object):
  
  def __init__(self, attrs={}):
    self.attributes = attrs
    if attrs.has_key('id'):
      self.id = attrs['id']
    else:
      self.computeId()
    self.read     = 0   # Flag lu/nonlu
    self.expanded = 0   # Flag description ouverte/ferme
    self.next = None    # Pointeur sur le suivant
  
  def computeId(self):
    self.id = md5.new(str(self.attributes)).hexdigest()
  
  def __str__(self):
    return repr(self.attributes['title'])
  
  def getTitle(self):
    return self.attributes['title']
  
  def getLink(self):
    return self.attributes['link']+self.attributes['groupid']+"/"+self.id+".html"
  
  def getGroupLink(self):
    return self.attributes['link']+self.attributes['groupid']+"/"
  
  def getDate(self):
    return self.attributes['date']
  
  def getAuthor(self):
    return self.attributes['author']
  
  def getGroup(self):
    return self.attributes['group']
  
  def getGroupId(self):
    return self.attributes['groupid']
  
  def getDescr(self):
    try:
      return self.attributes['description']
    except:
      return ''
  
  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class ForumsViewer(Frame):

  def __init__(self, parent, forum):
    self.root = parent
    self.forum = forum
    self.styles = self.forum.bouchot.styles
    Frame.__init__(self, parent)
    self.build_interface()
    self.update()
  
  def build_interface(self):
    fr = Frame(self)
    self.butmenu = Label(fr, text="Actions >", relief=FLAT, border=2)
    self.butmenu.bind("<Enter>", lambda e, s=self: s.butmenu.config(relief=RAISED))
    self.butmenu.bind("<Leave>", lambda e, s=self: s.butmenu.config(relief=FLAT))
    self.butmenu.bind("<1>", self.show_menu)
    self.butmenu.pack(side=LEFT)
    self.butclose = Label(fr, text=" X ", relief=FLAT, border=2)
    self.butclose.bind("<Enter>", lambda e, s=self: s.butclose.config(relief=RAISED))
    self.butclose.bind("<Leave>", lambda e, s=self: s.butclose.config(relief=FLAT))
    self.butclose.bind("<1>", self.quit)
    self.butclose.pack(side=RIGHT)
    self.label = Label(fr, font="Helvetica 10 bold", background=COLOR_TITLE)
    self.label.pack(side=LEFT, fill=X, expand=1)
    fr.pack(fill=X)
    self.menu = Menu(self.root, tearoff=0)
    self.menu.add_command(label="Marquer tous lus", command=self.markAllRead)
    self.menu.add_command(label="Drouler les non lus", command=self.expandAllUnread)
    self.menu.add_command(label="Ouvrir les non lus", command=self.openAll)
    self.menu.add_command(label="Drouler tout", command=self.expandAll)
    self.menu.add_command(label="Enrouler tout", command=self.collapseAll)
    self.ascending = IntVar()
    self.ascending.set(1)
    self.menu.add_checkbutton(label="Les plus rcents en haut", command=self.toggleSort, variable=self.ascending)
    self.menu.bind("<Escape>", self.hide_menu)
    if self.forum.options.val('docked'):
      btxt = "Dtacher"
    else:
      btxt = "Docker"
    self.menu.add_command(label=btxt, command=self.forum.toggleViewMode)
    self.menu.add_command(label="Configurer module...", command=self.showConfig)
    self.menu.add_command(label="Fermer", command=self.quit)
    self.images = { \
      'minus': PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'EYyPoivG614LAlg7ZZbxoR8UADs='),
      'plus':  PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'FIyPoiu2sJyCyoF7W3hxz850CFIA\nADs=')
    }
    self.list = ScrolledText(self, wrap=WORD)
    self.list.pack(side=RIGHT, fill=BOTH, expand=1)
    self.list.bind("<Enter>", lambda e, w=self: w.list.focus_set())
    self.list.bind("<Motion>", self.mouse_move)
    self.list.bind("<1>", self.mouse_click)
    self.list.bind("<3>", self.show_menu)
    self.list.bind("<Escape>", self.hide_menu)
  
  def show_menu(self, event):
    self.menu.post(event.x_root, event.y_root)
  
  def hide_menu(self, event=None, **kw):
    self.menu.unpost()
  
  def showConfig(self, event=None):
    self.forum.bouchot.pycc.showConfig(self.forum.name.replace(":", "_"))
  
  def update(self):
    """ Mise  jour du contenu de la fentre """
    self.setLbl(self.forum.title)
    self.list.config(state=NORMAL)
    self.list.config(cursor="arrow")
    # Effacement de l'ancien contenu
    self.list.delete(0.0, END)
    if len(self.forum.objects.keys()) > 0:
      # Configuration des styles
      self.apply_styles()
      item = self.forum.objects[self.forum.firstId]
      # Insertion des items
      while item is not None:
        self.insertItem(item)
        item = item.next
    else:
      self.list.insert(END, "Aucun lment trouv !")
    self.list.config(state=DISABLED)
  
  def apply_styles(self):
    apply_styles(self.list, self.styles, 'pinni', self.forum.bouchot.pycc.default_styles)
    self.list.tag_config('descr', lmargin1=20, lmargin2=20, borderwidth=1, relief=FLAT)
  
  def toggleSort(self, event=None):
    cursort = self.ascending.get()
    self.update()
  
  def mouse_move(self, event):
    self.unhilight()
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      if t[:4] == "url_" or t[:4] == "img_":
        self.forum.bouchot.pycc.setStatus(t[4:])
        self.setStyleTag(t, "urlhilight")
        self.list.config(cursor="hand2")
  
  def mouse_click(self, event):
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      if t[:4] == "url_":
        url = t[4:]
        self.forum.bouchot.pycc.openBrowser(url)
        self.setStyleTag(t, "urlvisit")
        self.forum.bouchot.pycc.setUrlRead(url)
      if t[:2] == "id":
        self.forum.markRead(self.forum.objects[t[2:]])
      if t[:4] == "img_":
        url = t[4:]
        self.forum.bouchot.pycc.getImageFromUrl(url)
        self.update()
    self.hide_menu()
  
  def setStyleTag(self, tag, style):
    tseq = self.list.tag_ranges(tag)
    cpt = 0
    while cpt < len(tseq):
      self.list.tag_add(style, tseq[cpt], tseq[cpt+1])
      cpt += 2
  
  def hilight(self, id):
    """ Mise en surbrillance d'un lien """
    beg, end = self.list.tag_ranges("id%s" % id)
    self.list.tag_add("urlhilight", beg, end)
    self.list.config(cursor="hand2")
  
  def unhilight(self, event=None):
    """ Enlve les surbrillances """
    self.list.tag_remove("urlhilight", 1.0, END)
    self.list.config(cursor="arrow")
    self.forum.bouchot.pycc.setStatus("")
  
  def openAll(self, event=None):
    for item in [i for i in self.forum.objects.values() if not i.read]:
      self.openItem(item)
  
  def openItem(self, item):
    """ Ouvre un lien dans un browser """
    self.forum.bouchot.pycc.openBrowser(item.getLink())
    self.forum.markRead(item)
  
  def insertItem(self, item):
    self.list.config(state=NORMAL)
    if item.expanded:
      sign = self.images['minus']
    else:
      sign = self.images['plus']
    if self.ascending.get():
      self.list.mark_set(INSERT, 0.0)
    else:
      self.list.mark_set(INSERT, END)
    self.list.image_create(INSERT, image=sign, align=CENTER)
    self.list.insert(INSERT, " ")
    self.list.tag_add("fold_%s" % item.id, "insert-3c", "insert-1c")
    self.list.tag_bind("fold_%s" % item.id, "<ButtonRelease-1>", lambda e, w=self, i=item: w.toggle(i))
    self.list.insert(INSERT, item.getDate(), "time")
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.getAuthor(), "login")
    self.list.insert(INSERT, " ")
    t = "groupid%s" % item.getGroupId()
    self.list.insert(INSERT, item.getGroup(), t)
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getGroupLink())
    self.list.insert(INSERT, " : ")
    t = "id%s" % item.id
    self.list.insert(INSERT, item.getTitle(), t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getLink())
    if item.expanded:
      msg, tags = self.forum.parseDescr(item.getDescr())
      try:
        msg = toUnicode(msg, self.forum.options.val('encoding'))
      except:
        pass
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      for tag in tags.keys():
        for range in tags[tag]:
          self.list.tag_add(tag, "%s+%s chars" % (start, range[0]), "%s+%s chars" % (start, range[1]))
        if tag[:4] == 'url_':
          if tag in self.forum.bouchot.pycc.urlread:
            self.setStyleTag(tag, 'urlvisit')
      if tags.has_key('img'):
        if self.forum.options.val('display_img'):
          myranges = tags['img']
          myranges.sort()
          myranges.reverse()
          for range in myranges:
            idx_start = "%s+%s chars" % (start, range[0])
            idx_end = "%s+%s chars" % (start, range[1])
            url = self.list.get(idx_start, idx_end)[7:-1]
            imgname = md5.new(url).hexdigest()
            self.list.delete(idx_start, idx_end)
            try:
              if not self.images.has_key(imgname):
                self.images[imgname] = get_image(os.path.join(self.forum.bouchot.pycc.config_path, "cache", imgname))
              self.list.image_create(idx_start, image=self.images[imgname], align=CENTER)
            except:
              log.warning("%s: Image '%s' introuvable ou fichier corrompu" % (self.forum.name, imgname))
              self.list.insert(idx_start, "## Erreur ## Image: %s" % imgname, ("img", "img_%s" % url))
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)
  
  def setLbl(self, text):
    self.label.config(text=text)
  
  def markRead(self, item):
    # Marque un lien comme lu
    self.setStyleTag("url_"+item.getLink(), "urlvisit")
  
  def markAllRead(self, event=None):
    for item in self.forum.objects.values():
      self.forum.markRead(item)
  
  def expandAllUnread(self, event=None):
    for item in self.forum.objects.values():
      item.expanded = not item.read
      if self.forum.options.val('read_at_exp'):
        self.forum.markRead(item)
    self.update()
  
  def expandAll(self, event=None):
    for item in self.forum.objects.values():
      if self.forum.options.val('read_at_exp'):
        self.forum.markRead(item)
      item.expanded = 1
    self.update()
  
  def collapseAll(self, event=None):
    for item in self.forum.objects.values():
      item.expanded = 0
    self.update()
  
  def showItem(self, item):
    item.expanded = 1
    if self.forum.options.val('read_at_exp'):
      self.forum.markRead(item)
    self.update()
  
  def hideItem(self, item):
    item.expanded = 0
    self.update()
  
  def toggle(self, item):
    if item.expanded:
      self.hideItem(item)
    else:
      self.showItem(item)
    self.list.see(self.list.tag_ranges('id%s' % item.id)[0])
  
  def quit(self, event=None):
    self.forum.closeViewer()
  

class Forums(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = self.options.val('descr')
  
  def build_viewer(self, parent):
    self.viewer = ForumsViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = ForumsOptions(path=self.config_path)
  
  def unload(self):
    # AFR ?
    pass
  
  def parse(self, file):
    tmplist = []
    text = file.read()
    encoding = "iso-8859-15"
    text = toUnicode(text, encoding)
    """
    try:
      text = text.encode(encoding)
    except:
      log.warning("%s: le texte du backend passe pas avec l'encoding spcifi (%s)" % (self.name, encoding))
    """
    self.title = "DLFP : Forums..."
    start = text.find('<div class="titlediv">')
    end = 0
    while start > end:
      attrs = {}
      start = text.find('<a href="', start)
      end = text.find('/">', start)
      if end < start: break
      attrs['groupid'] = text[start+9:end]
      start = text.find('</a>', end)
      if start < end: break
      attrs['group'] = text[end+3:start].replace("\n", " ").strip()
      end = text.find('</h1>', start)
      if end < start: break
      attrs['title'] = text[start+7:end].replace("\n", " ").strip()
      start = text.find('/">', end)
      if start < end: break
      end = text.find('</a>', start)
      if end < start: break
      attrs['author'] = text[start+3:end].replace("\n", " ").strip()
      start = text.find('\n', end)
      if start < end: break
      end = text.find('<script', start)
      if end < start: break
      attrs['date'] = text[start+1:end].replace("\n", " ").strip()
      start = text.find(')</script>', end)
      if start < end: break
      attrs['id'] = text[end+44:start]
      start = text.find('<div class="bodydiv">', end)
      if start < end: break
      end = text.find('<p>', start)
      if end < start: break
      attrs['description'] = text[start+22:end].replace("\n", " ").strip()
      attrs['link'] = "http://%s%s" % (self.bouchot.options.val('http_server'), self.options.val('backend'))
      newitem = ForumItem(attrs)
      newitem.read = self.isRead(newitem.id)
      tmplist.append(newitem)
      start = text.find('<div class="titlediv">', end)
    tmplist.reverse()
    return tmplist
  
  def process(self, items=[]):
    for forumitem in items:
      if self.firstId is None:
        self.firstId = forumitem.id
      if forumitem.id not in self.objects.keys():
        log.debug("%s: nouveau forumitem: %s" % (self.name, forumitem.id))
        self.objects[forumitem.id] = forumitem
        if self.lastId is not None:
          self.objects[self.lastId].next = forumitem
        self.lastId = forumitem.id
        if self.viewer_open:
          self.viewer.insertItem(forumitem)
        self.bouchot.setState(STATE_NEW)
    self.updateLed()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def markRead(self, item):
    if not item.read:
      try:
        f = open(os.path.join(self.config_path, "read"), 'a')
        f.write(item.id+"\n")
        f.close()
      except:
        log.warning("%s: Ecriture dans le fichier 'read' impossible" % self.name)
    item.read = 1
    if self.viewer_open:
      self.viewer.markRead(item)
    self.updateLed()
  
  def isRead(self, id):
    try:
      f = open(os.path.join(self.config_path, "read"), 'r')
      readlist = f.read().split("\n")
      f.close()
    except:
      log.warning("%s: Lecture dans le fichier 'read' impossible" % self.name)
      return 0
    return id in readlist
  
  def parseDescr(self, text):
    if self.options.val('tags_encoded'):
      slip_style = "Tags encoded"
    else:
      slip_style = "Tags not encoded"
    text = htmlParser.pre_parse(text, slip_style, self.options.val('encoding'))
    if self.options.val('display_img'):
      start = text.find("<img")
      end = text.find(">", start)
      while start > -1 and end > start:
        url = getAttr("src", text[start:end])
        text = text[:start] + "<img>[image:" + url + "]</img>" + text[end+1:]
        start = text.find("<img", start+5)
        end = text.find(">", start)
        self.bouchot.pycc.getImageWithCache(url)
    msg, tags = htmlParser.html_parse(text, encoding=self.options.val('encoding'))
    return (msg, tags)
  