# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module modules.module (ahah bis)
##   Classe-mre pour tous les modules.
## ----------------------------------------------------------------------

import os, os.path
from urllib2 import HTTPError, URLError

import logging
log = logging.getLogger('pycc')

import sys, os.path
sys.path.append('..')

from Tkinter import *
from constants import *
from cointklib.basicwidgets import *
from cointklib.widgets import OptionsEditor
from cointklib.dialogs import CoinTopLevel
from coinlib.threads import GenericThread
from coinlib.persistants import OptionSet, Option
from coinlib.utils import open_url
from coinlib.parsers import getAttr

class ModuleOptions(OptionSet):
  
  def setObjects(self):
    self._objects = {
      'descr':      Option("Description (nom pour affichage)", "Generic Module"),
      'short_name': Option("Nom court (pour la diode)", "GM"),
      'backend':    Option("URL backend", "/"),
      'encoding':   Option("Encoding du backend", "iso-8859-15"),
      'docked':     Option("Docker dans le pinni", 0, TYPE_BOOL),
      'toedit':     Option("Editables", {"Gnral": ['backend']})
    }
  

class Module(object):
  
  def __init__(self, bouchot, name):
    self.bouchot = bouchot   # Bouchot d'appartenance
    self.name = "%s:%s" % (bouchot.name, name)
    self.config_path = os.path.join(self.bouchot.config_path, name)
    self.loaded = 0
    self.load_config()
  
  def create_config(self):
    os.mkdir(self.config_path)
  
  def load_config(self):
    log.debug("Chargement config module %s" % self.name)
    self.options = ModuleOptions(path=self.config_path)  # Override in derivated classes
  
  def save_config(self):
    log.debug("Sauvegarde config module %s" % self.name)
    self.options.save()
  
  def apply_config(self):
    pass # Override in derivated classes
  
  def config_frame(self, parent):
    return OptionsEditor(parent, self.options, extra_arg=self.bouchot)
  
  def build_viewer(self, parent):
    self.viewer = Frame(parent)  # Override in derivated classes
  
  def apply_styles(self):
    pass # Override in derivated classes
  
  def add_led(self):
    log.debug("Ajout de la diode pour %s" % self.name)
    try:
      shname = self.options.val('short_name')
    except PyccError:
      shname = self.name[:1]
    self.led = Led(self.bouchot.ledbar, shname)
    self.led.pack(side='left')
    for i in range(1,4):
      self.led._led.bind("<ButtonRelease-%s>"%i, lambda e, s=self, act=i: s.led_click(act))
    if self.bouchot.pycc.options.val('emul_3but'):
      self.led._led.bind("<Control-ButtonRelease-1>", lambda e, s=self: s.led_click(2))
  
  def initialize(self):
    log.debug("Initialisation du module %s" % self.name)
    self.objects = {}
    try:
      delay = self.options.val('check_delay')
    except PyccError:
      delay = 300 # refresh  5 minutes par dfaut
    self.thread = GenericThread(self.name, delay)
    self.thread.process = self.action
    self.thread._initialize(delay)
    self.add_led()
    self.viewer = None
    self.viewer_open = 0
    self.lastModif = ''
    self.init()  # Initialisation spcifique
    self.loaded = 1
  
  def init(self):
    pass # Override
  
  def openViewer(self):
    if self.viewer_open:
      self.viewer.focus_set()
      return
    if self.viewer is None:
      if self.options.val('docked'):
        parent = self.bouchot.pw.add("%s:view" % self.name, min=.1, size=.4)
        self.bouchot.pw.updatelayout()
      else:
        parent = CoinTopLevel(self.bouchot.root, self.options.val('descr'))
        parent.protocol("WM_DELETE_WINDOW", self.closeViewer)
      self.build_viewer(parent)
    self.show()
  
  def closeViewer(self):
    self.hide()
    self.viewer.root.destroy()
    self.viewer = None
  
  def show(self):
    self.viewer.pack(fill=BOTH, expand=1)
    self.viewer_open = 1
  
  def hide(self):
    # self.viewer.pack_forget()
    if self.options.val('docked'):
      self.bouchot.pw.delete("%s:view" % self.name)
      self.bouchot.pw.updatelayout()
    self.viewer_open = 0
  
  def toggleViewMode(self):
    self.closeViewer()
    actual = self.options.val('docked')
    self.options.set('docked', not actual)
    self.openViewer()
  
  def start(self):
    log.info("Dmarrage %s" % self.name)
    self.thread.start()
  
  def setState(self, state, msg=''):
    self.state = state
    if state == STATE_OK:
      self.led.clear()
    else:
      self.led.setState(state, msg)
      if state != STATE_HTTP:
        self.bouchot.setState(state)
  
  def get_http_params(self):
    # Override
    # Retourne: ua, cookie et data
    return ('', '', None)
  
  def unchanged(self, conn):
    last = conn.info().getheader('Last-Modified')
    if last and self.lastModif == last:
      return 1
    self.lastModif = last
    return 0
  
  def getEncoding(self, line):
    if line.find("<?xml") >= 0:
      try:
        encoding = getAttr("encoding", line)
        self.options.set("encoding", encoding)
        log.info("%s: Encoding dtect dans le backend: %s" % (self.name, encoding))
        return encoding
      except:
        pass
    encoding = self.options.val('encoding')
    log.info("%s: Pas d'encoding dans le backend, valeur par dfaut: %s" % (self.name, encoding))
    return encoding
  
  def parse(self, file):
    log.debug("%s: mthode parse gnrique" % self.name)
    print file.read()
    return []
  
  def action(self):
    self.setState(STATE_HTTP)
    server = self.bouchot.options.val('http_server')
    url = self.options.val('backend')
    ua, cookie, data = self.get_http_params()
    headers = {}
    if self.lastModif:
      headers['If-Modified-Since'] = self.lastModif
    self.error = ""
    try:
      file = open_url(server, url, self.bouchot.pycc.options, ua, cookie, data, headers)
    except HTTPError, value:
      if value.code == 302 or value.code == 304:
        self.setState(STATE_OK)
        log.debug("%s - %s" % (url, value))
      else:
        self.error = str(value)
        self.setState(STATE_ERROR, self.error)
        log.error("Erreur HTTP sur %s - %s:\n    %s" % (self.name, url, self.error))
      return
    except URLError, value:
      self.error = str(value.reason)
      self.setState(STATE_ERROR, self.error)
      log.error("%s: Erreur en essayant de rapatrier %s:\n    %s" % (self.name, url, self.error))
      return
    except:
      self.error = "Y'a eu un truc zarb sur %s" % url
      self.setState(STATE_ERROR, self.error)
      log.error("%s: Erreur http inconnue sur %s" % (self.name, url))
      return
    if self.unchanged(file):
      file.close()
      log.debug("%s n'a pas chang" % url)
      return
    object_list = []
    try:
      object_list = self.parse(file)
    except:
      self.error = "Ca a plant bizarrement"
      self.setState(STATE_ERROR, self.error)
      log.error("%s: Erreur inconnue en parsant %s" % (self.name, url))
    else:
      if self.error:
        self.setState(STATE_WARNING, self.error)
        log.warning("Erreur de parsing sur %s:\n    %s" % (self.name, self.error))
      else:
        self.setState(STATE_OK)
    file.close()
    if object_list:
      self.process(object_list)  # Ce qu'on fait des objets parss
  
  def process(self, object_list=[]):
    log.debug("Je fais le process de %s" % self.name)
    # Abstract --  implmenter dans les modules drivs
  
  def led_click(self, action):
    if action == 1:
      if self.viewer_open:
        self.closeViewer()
      else:
        self.openViewer()
    elif action == 2:
      self.led.showMsg(self.bouchot.pw.interior(), "Dernire erreur sur %s" % self.name)
    else:
      self.thread.restart()
  
  def unload(self):
    pass  # Override
  
  def quit(self):
    if not self.loaded:
      log.debug("%s.quit(): module dj dcharg" % self.name)
      return
    if self.viewer is not None:
      self.closeViewer()
    self.thread.stop()
    self.unload()
    self.led.destroy()
    self.loaded = 0
  