# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module rssreader
##   Dfinition du module de bouchot RSSReader :
##   monitoring d'un feed rss (pour les news, etc.)
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

import xml.dom.minidom

from constants import *
from modules.module import *
from coinlib.persistants import OptionSet, Option
from coinlib.utils import *
from coinlib.parsers import *

def build_module(parent, name):
  return RSSReader(parent, name)

import re
regexp_newsid = re.compile("^.*/([0-9]+)/([0-9]+)/([0-9]+)/([^/,.]+)[,.][^/]*$")

class RSSReaderOptions(OptionSet):
  """ Options du module RSSReader
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "RSSReader"),
      'short_name':   Option("Nom court (pour la diode)", "N"),
      'backend':      Option("URL du fichier de backend", ""),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'encoding':     Option("Encoding du backend", "iso-8859-15"),
      'check_delay':  Option("Nombre de secondes entre 2 checks du backend", 300, TYPE_INT),
      'max_history':  Option("Nombre maximum d'lments  conserver", 50, TYPE_INT),
      'read_at_exp':  Option("Marquer lu ds l'ouverture dans le viewer", 0, TYPE_BOOL),
      'slip_crade':   Option("Slip pas propre", 0, TYPE_BOOL),
      'keep_cr':      Option("Conserver les retours  la ligne", 1, TYPE_BOOL),
      'tags_encoded': Option("Tags encods", 0, TYPE_BOOL),
      'find_author':  Option("Trouver l'auteur dans l'url (journaux dlfp)", 0, TYPE_BOOL),
      'find_date':    Option("Trouver la date dans l'url (news dlfp)", 0, TYPE_BOOL),
      'find_id':      Option("Trouver l'ID dans l'url", 0, TYPE_BOOL),
      'display_img':  Option("Dtecter et afficher les images", 1, TYPE_BOOL),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'short_name', 'backend', 'check_delay', 'max_history', 'read_at_exp', 'docked' ],
          "Parsing": [ 'tags_encoded', 'keep_cr', 'slip_crade', 'find_date', 'find_author', 'find_id', 'display_img', 'encoding' ]
        } )
    }


class RSSItem(object):
  
  def __init__(self, attrs={}):
    self.attributes = attrs
    if attrs.has_key('id'):
      self.id = attrs['id']
    else:
      self.computeId()
    self.read     = 0   # Flag lu/nonlu
    self.expanded = 0   # Flag description ouverte/ferme
    self.next = None    # Pointeur sur le suivant
  
  def computeId(self):
    self.id = md5.new(str(self.attributes)).hexdigest()
  
  def getTitle(self):
    if not self.attributes.has_key('title'):
      return u'[pas de titre]'
    return self.attributes['title']
  
  def getLink(self):
    for lkey in ['link', 'guid']:
      if self.attributes.has_key(lkey):
        return self.attributes[lkey]
      else:
        continue
    return ''
  
  def getDate(self):
    for dkey in ['date', 'pubDate', 'dc:date']:
      if self.attributes.has_key(dkey):
        return self.attributes[dkey]
      else:
        continue
    return ''
  
  def getAuthor(self):
    for akey in ['author', 'creator', 'dc:creator']:
      if self.attributes.has_key(akey):
        return self.attributes[akey]
      else:
        continue
    return ''
  
  def getDescr(self):
    try:
      return self.attributes['description']
    except:
      return ''
  
  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class RSSViewer(Frame):

  def __init__(self, parent, rssreader):
    self.root = parent
    self.rssreader = rssreader
    self.styles = self.rssreader.bouchot.styles
    Frame.__init__(self, parent)
    self.build_interface()
    self.update()
  
  def build_interface(self):
    fr = Frame(self)
    self.butmenu = Label(fr, text="Actions >", relief=FLAT, border=2)
    self.butmenu.bind("<Enter>", lambda e, s=self: s.butmenu.config(relief=RAISED))
    self.butmenu.bind("<Leave>", lambda e, s=self: s.butmenu.config(relief=FLAT))
    self.butmenu.bind("<1>", self.show_menu)
    self.butmenu.pack(side=LEFT)
    self.butclose = Label(fr, text=" X ", relief=FLAT, border=2)
    self.butclose.bind("<Enter>", lambda e, s=self: s.butclose.config(relief=RAISED))
    self.butclose.bind("<Leave>", lambda e, s=self: s.butclose.config(relief=FLAT))
    self.butclose.bind("<1>", self.quit)
    self.butclose.pack(side=RIGHT)
    self.label = Label(fr, font="Helvetica 10 bold", background=COLOR_TITLE)
    self.label.pack(side=LEFT, fill=X, expand=1)
    fr.pack(fill=X)
    self.menu = Menu(self.root, tearoff=0)
    self.menu.add_command(label="Marquer tous lus", command=self.markAllRead)
    self.menu.add_command(label="Drouler les non lus", command=self.expandAllUnread)
    self.menu.add_command(label="Ouvrir les non lus", command=self.openAll)
    self.menu.add_command(label="Drouler tout", command=self.expandAll)
    self.menu.add_command(label="Enrouler tout", command=self.collapseAll)
    self.ascending = IntVar()
    self.ascending.set(1)
    self.menu.add_checkbutton(label="Les plus rcents en haut", command=self.toggleSort, variable=self.ascending)
    self.menu.bind("<Escape>", self.hide_menu)
    if self.rssreader.options.val('docked'):
      btxt = "Dtacher"
    else:
      btxt = "Docker"
    self.menu.add_command(label=btxt, command=self.rssreader.toggleViewMode)
    self.menu.add_command(label="Configurer module...", command=self.showConfig)
    self.menu.add_command(label="Fermer", command=self.quit)
    self.images = { \
      'minus': PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'EYyPoivG614LAlg7ZZbxoR8UADs='),
      'plus':  PhotoImage(
                  data='R0lGODdhCQAJAPEAAAAAAH9/f////wAAACwAAAAACQAJAAAC' \
                  'FIyPoiu2sJyCyoF7W3hxz850CFIA\nADs=')
    }
    self.list = ScrolledText(self, wrap=WORD)
    self.list.pack(side=RIGHT, fill=BOTH, expand=1)
    self.list.bind("<Enter>", lambda e, w=self: w.list.focus_set())
    self.list.bind("<Motion>", self.mouse_move)
    self.list.bind("<1>", self.mouse_click)
    self.list.bind("<3>", self.show_menu)
    self.list.bind("<Escape>", self.hide_menu)
  
  def show_menu(self, event):
    self.menu.post(event.x_root, event.y_root)
  
  def hide_menu(self, event=None, **kw):
    self.menu.unpost()
  
  def showConfig(self, event=None):
    self.rssreader.bouchot.pycc.showConfig(self.rssreader.name.replace(":", "_"))
  
  def update(self):
    """ Mise  jour du contenu de la fentre """
    self.setLbl(self.rssreader.title)
    self.list.config(state=NORMAL)
    self.list.config(cursor="arrow")
    # Effacement de l'ancien contenu
    self.list.delete(0.0, END)
    if len(self.rssreader.objects.keys()) > 0:
      # Configuration des styles
      self.apply_styles()
      item = self.rssreader.objects[self.rssreader.firstId]
      # Insertion des items
      while item is not None:
        self.insertItem(item)
        item = item.next
    else:
      self.list.insert(END, "Aucun lment trouv !")
    self.list.config(state=DISABLED)
  
  def apply_styles(self):
    apply_styles(self.list, self.styles, 'pinni', self.rssreader.bouchot.pycc.default_styles)
    self.list.tag_config('descr', lmargin1=20, lmargin2=20, borderwidth=1, relief=FLAT)
  
  def toggleSort(self, event=None):
    cursort = self.ascending.get()
    self.update()
  
  def mouse_move(self, event):
    self.unhilight()
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      if t[:4] == "url_" or t[:4] == "img_":
        self.rssreader.bouchot.pycc.setStatus(t[4:])
        self.setStyleTag(t, "urlhilight")
        self.list.config(cursor="hand2")
  
  def mouse_click(self, event):
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      if t[:4] == "url_":
        url = t[4:]
        self.rssreader.bouchot.pycc.openBrowser(url)
        self.setStyleTag(t, "urlvisit")
        self.rssreader.bouchot.pycc.setUrlRead(url)
      if t[:2] == "id":
        self.rssreader.markRead(self.rssreader.objects[t[2:]])
      if t[:4] == "img_":
        url = t[4:]
        self.rssreader.bouchot.pycc.getImageFromUrl(url)
        self.update()
    self.hide_menu()
  
  def setStyleTag(self, tag, style):
    tseq = self.list.tag_ranges(tag)
    cpt = 0
    while cpt < len(tseq):
      self.list.tag_add(style, tseq[cpt], tseq[cpt+1])
      cpt += 2
  
  def hilight(self, id):
    """ Mise en surbrillance d'un lien """
    beg, end = self.list.tag_ranges("id%s" % id)
    self.list.tag_add("urlhilight", beg, end)
    self.list.config(cursor="hand2")
  
  def unhilight(self, event=None):
    """ Enlve les surbrillances """
    self.list.tag_remove("urlhilight", 1.0, END)
    self.list.config(cursor="arrow")
    self.rssreader.bouchot.pycc.setStatus("")
  
  def openAll(self, event=None):
    for item in [i for i in self.rssreader.objects.values() if not i.read]:
      self.openItem(item)
  
  def openItem(self, item):
    """ Ouvre un lien dans un browser """
    self.rssreader.bouchot.pycc.openBrowser(item.getLink())
    self.rssreader.markRead(item)
  
  def insertItem(self, item):
    self.list.config(state=NORMAL)
    if item.expanded:
      sign = self.images['minus']
    else:
      sign = self.images['plus']
    if self.ascending.get():
      self.list.mark_set(INSERT, 0.0)
    else:
      self.list.mark_set(INSERT, END)
    self.list.image_create(INSERT, image=sign, align=CENTER)
    self.list.insert(INSERT, " ")
    self.list.tag_add("fold_%s" % item.id, "insert-3c", "insert-1c")
    self.list.tag_bind("fold_%s" % item.id, "<ButtonRelease-1>", lambda e, w=self, i=item: w.toggle(i))
    self.list.insert(INSERT, item.getDate(), "time")
    self.list.insert(INSERT, " ")
    self.list.insert(INSERT, item.getAuthor(), "login")
    self.list.insert(INSERT, " ")
    t = "id%s" % item.id
    self.list.insert(INSERT, item.getTitle(), t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getLink())
    if item.expanded:
      text = toUnicode(item.getDescr(), self.rssreader.options.val('encoding'))
      msg, tags = self.rssreader.parseDescr(text)
      start = self.list.index("insert")
      self.list.insert(INSERT, msg+"\n\n", 'descr')
      for tag in tags.keys():
        for range in tags[tag]:
          self.list.tag_add(tag, "%s+%s chars" % (start, range[0]), "%s+%s chars" % (start, range[1]))
        if tag[:4] == 'url_':
          if tag in self.rssreader.bouchot.pycc.urlread:
            self.setStyleTag(tag, 'urlvisit')
      if tags.has_key('img'):
        if self.rssreader.options.val('display_img'):
          myranges = tags['img']
          myranges.sort()
          myranges.reverse()
          for range in myranges:
            idx_start = "%s+%s chars" % (start, range[0])
            idx_end = "%s+%s chars" % (start, range[1])
            url = self.list.get(idx_start, idx_end)[7:-1]
            imgname = md5.new(url).hexdigest()
            self.list.delete(idx_start, idx_end)
            try:
              if not self.images.has_key(imgname):
                self.images[imgname] = get_image(os.path.join(self.rssreader.bouchot.pycc.config_path, "cache", imgname))
              self.list.image_create(idx_start, image=self.images[imgname], align=CENTER)
            except:
              log.warning("%s: Image '%s' introuvable ou fichier corrompu" % (self.rssreader.name, imgname))
              self.list.insert(idx_start, "## Erreur ## Image: %s" % imgname, ("img", "img_%s" % url))
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)
  
  def setLbl(self, text):
    self.label.config(text=text)
  
  def markRead(self, item):
    # Marque un lien comme lu
    self.setStyleTag("url_"+item.getLink(), "urlvisit")
  
  def markAllRead(self, event=None):
    for item in self.rssreader.objects.values():
      self.rssreader.markRead(item)
  
  def expandAllUnread(self, event=None):
    for item in self.rssreader.objects.values():
      item.expanded = not item.read
      if self.rssreader.options.val('read_at_exp'):
        self.rssreader.markRead(item)
    self.update()
  
  def expandAll(self, event=None):
    for item in self.rssreader.objects.values():
      if self.rssreader.options.val('read_at_exp'):
        self.rssreader.markRead(item)
      item.expanded = 1
    self.update()
  
  def collapseAll(self, event=None):
    for item in self.rssreader.objects.values():
      item.expanded = 0
    self.update()
  
  def showItem(self, item):
    item.expanded = 1
    if self.rssreader.options.val('read_at_exp'):
      self.rssreader.markRead(item)
    self.update()
  
  def hideItem(self, item):
    item.expanded = 0
    self.update()
  
  def toggle(self, item):
    if item.expanded:
      self.hideItem(item)
    else:
      self.showItem(item)
    self.list.see(self.list.tag_ranges('id%s' % item.id)[0])
  
  def quit(self, event=None):
    self.rssreader.closeViewer()
  

def getText(nodelist, encoding='iso-8859-15'):
  rc = ""
  for node in nodelist:
    if node.nodeType == node.TEXT_NODE:
      rc = rc + node.data
  return toUnicode(rc, encoding)

def getXML(nodelist, encoding='iso-8859-15'):
  rc = ""
  for node in nodelist:
    rc = rc + node.toxml()
  rc = rc.replace('<![cdata[','')
  rc = rc.replace('<![CDATA[','')
  rc = rc.replace(']]>','')
  return toUnicode(rc, encoding)
  """try:
    return rc.encode('iso-8859-15')
  except:
    return rc"""

def handle_stinking_underpants(text):
  msg = text.replace("&lt;", "<")
  start = msg.find("&")
  end = 0
  res = ""
  while start >= 0:
    res += msg[end:start]
    if msg[start+1] == " ":
      res += "&amp;"
      end = start+1
      start = msg.find("&", end)
      continue
    end = msg.find(";", start)
    if end < start:
      end = start
      break
    entity = msg[start+1:end]
    for c in entity:
      if c.lower() not in "azertyuiopqsdfghjklmwxcvbn1234567890_-#":
        entity = "amp;%s" % entity
        break
    res += "&%s;" % entity
    end += 1
    start = msg.find("&", end)
  res += msg[end:]
  return res


class RSSReader(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = self.options.val('descr')
    self.rssElements = ['title', 'link', 'guid', 'description', 'pubDate', 'date', 'author', 'dc:date', 'dc:creator']
    # self.img_urls = {}
  
  def build_viewer(self, parent):
    self.viewer = RSSViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = RSSReaderOptions(path=self.config_path)
  
  def unload(self):
    # AFR ?
    pass
  
  def parse(self, file):
    tmplist = []
    text = file.read()
    """ # Gestion d'erreurs un peu pourrie (dpend trop du serveur)
    firstline = text.strip().split("\n")[0].strip()
    print "---- myfirstline:", firstline
    if firstline[:3].lower() == "err":
      self.error = "Le serveur a rpondu:\n%s" % firstline
      return []
    """
    if self.options.val('slip_crade'):
      text = handle_stinking_underpants(text)
    encoding = self.getEncoding(text)
    """try:
      text = text.decode(encoding)
    except:
      log.warning("%s: le texte du backend passe pas avec l'encoding spcifi (%s)" % (self.name, encoding))
    """
    try:
      dom = xml.dom.minidom.parseString(text)
    except:
      self.error = "Erreur de parsing XML (fichier rss moisi ?)"
      return []
    channel = dom.getElementsByTagName("channel")[0]
    self.title = getText(channel.getElementsByTagName("title")[0].childNodes, encoding)
    if not self.options.val('keep_cr'):
      self.title = self.title.replace('\n', ' ')
    for item in dom.getElementsByTagName("item"):
      attrs = {}
      for elem in self.rssElements:
        domlist = item.getElementsByTagName(elem)
        if len(domlist):
          if elem == 'description' or elem == 'title':
            msg = getXML(domlist[0].childNodes, encoding)
            attrs[elem] = msg
          else:
            attrs[elem] = getText(domlist[0].childNodes, encoding)
            if elem == 'link':
              link = attrs[elem]
              if self.options.val('find_date'):
                newsid = regexp_newsid.match(link)
                if self.options.val('find_id'):
                  attrs['id'] = newsid.groups()[3]
                attrs['date'] = "%s/%s/%s" % (newsid.groups()[2], newsid.groups()[1], newsid.groups()[0])
              if self.options.val('find_author'):
                pos1 = link.find("~")
                pos2 = link.find("/", pos1)
                attrs['author'] = link[pos1+1:pos2]
                if self.options.val('find_id'):
                  attrs['id'] = link[pos2+1:-5]
          if not self.options.val('keep_cr'):
            attrs[elem] = attrs[elem].replace('\n', ' ')
      newitem = RSSItem(attrs)
      newitem.read = self.isRead(newitem.id)
      tmplist.append(newitem)
    dom.unlink()
    tmplist.reverse()
    return tmplist
  
  def process(self, items=[]):
    for rssitem in items:
      if self.firstId is None:
        self.firstId = rssitem.id
      if rssitem.id not in self.objects.keys():
        log.debug("%s: nouvel rssitem: %s" % (self.name, rssitem.id))
        self.objects[rssitem.id] = rssitem
        if self.lastId is not None:
          self.objects[self.lastId].next = rssitem
        self.lastId = rssitem.id
        if self.viewer_open:
          self.viewer.insertItem(rssitem)
        self.bouchot.setState(STATE_NEW)
    self.updateLed()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def markRead(self, item):
    if not item.read:
      try:
        f = open(os.path.join(self.config_path, "read"), 'a')
        f.write(item.id+"\n")
        f.close()
      except:
        log.warning("%s: Ecriture dans le fichier 'read' impossible" % self.name)
    item.read = 1
    if self.viewer_open:
      self.viewer.markRead(item)
    self.updateLed()
  
  def isRead(self, id):
    try:
      f = open(os.path.join(self.config_path, "read"), 'r')
      readlist = f.read().split("\n")
      f.close()
    except:
      log.warning("%s: Lecture dans le fichier 'read' impossible" % self.name)
      return 0
    return id in readlist
  
  def parseDescr(self, text):
    if self.options.val('tags_encoded'):
      slip_style = "Tags encoded"
    else:
      slip_style = "Tags not encoded"
    text = htmlParser.pre_parse(text, slip_style, self.options.val('encoding'))
    if self.options.val('display_img'):
      start = text.find("<img")
      end = text.find(">", start)
      while start > -1 and end > start:
        url = getAttr("src", text[start:end])
        text = text[:start] + "<img>[image:" + url + "]</img>" + text[end+1:]
        start = text.find("<img", start+5)
        end = text.find(">", start)
        self.bouchot.pycc.getImageWithCache(url)
    msg, tags = htmlParser.html_parse(text, encoding=self.options.val('encoding'))
    return (msg, tags)
  