# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module Pinnipede
##   Dfinition du Pinnipede Teletype pour le module Tribune
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os.path
sys.path.append('../..')

from Tkinter import *
from constants import *
from ScrolledText import ScrolledText
from coinlib.threads import GenericThread
from coinlib.utils import apply_styles, get_image

import types
_unicode = types.UnicodeType

class MutexWriter(GenericThread):
  """ Dfinit un thread qui se charge des critures dans le pinnipde,
      ce afin de pouvoir les grer de manire asynchrone
  """
  def __init__(self, pinni):
    log.debug("## Entering MutexWriter.__init__")
    GenericThread.__init__(self, "%s:MutexWriter" % pinni.trib.name, type="queue")
    self.pinni = pinni
    self.trib = self.pinni.trib
    self.text = self.pinni._pinni
  
  def process(self, queueitem):
    log.debug("## Entering MutexWriter.process")
    action_type, itag, args = queueitem
    item, start, end, tags, tobescrolled = args
    
    #== Calcul des positions  partir d'un tag
    if itag:
      try:
        pos = self.text.tag_ranges(itag)
        start = pos[0]
        end = pos[1]
      except:
        log.error("%s: tag '%s' inexistant - impossible de trouver les bornes" % (self.name, itag))
        return
    
    #== Calcul de position pour ajout
    if action_type == PINNI_ADD:
      start = self.text.index("end-1 chars")
    
    #== Protection contre les cas d'erreur Tkinter
    if start is None:
      log.error("%s: Erreur d'index - action: %s item: %s" % (self.name, action_type, item))
      self.write(queueitem)
      return
    
    #== Ajout d'un tag
    if action_type == PINNI_SETSTYLE:
      try:
        self.text.tag_add(item, start, end)
      except:
        # Tk se vautre parfois ici
        log.warning("%s: Vautrage Tk, sur SetStyle %s (%s, %s)" % (self.name, item, start, end))
      return
    
    #== Insertion d'image
    if action_type == PINNI_IMAGE:
      self.text.config(state='normal')
      try:
        self.text.image_create(end, image=item, align=CENTER)
        self.text.tag_remove(itag, start, end)
      except:
        self.text.config(state='disabled')
        return
      for tag, tag_ranges in tags.items():
        for range in tag_ranges:
          self.text.tag_add(tag, "%s+%s chars" % (end, range[0]), "%s+%s chars" % (end, range[1]))
    
    #== Effacement
    if action_type in (PINNI_DELETE, PINNI_REPLACE, PINNI_IMAGE):
      try:
        self.text.config(state='normal')
        self.text.delete(start, end)
        self.text.config(state='disabled')
        if action_type == PINNI_DELETE and itag and self.text.tag_ranges(itag):
          self.write(queueitem)
        if action_type != PINNI_REPLACE:
          return
      except TclError:
        log.error("%s: Erreur Tcl sur delete dans le pinni" % self.name)
        self.text.config(state='disabled')
        self.write(queueitem)
        return
    
    #== Insertion
    if action_type == PINNI_INSERT:
      itag = ""
    
    #== Insertion du texte
    if type(item) == _unicode:
      text = item
    else:
      try:
        text = item.decode(self.trib.options.val('encoding'))
      except:
        log.warning("%s: Erreur unicode sur %s" % (self.name, repr(item)))
        text = item
    self.text.config(state='normal')
    try:
      self.text.insert(start, text, itag)
    except SystemError:
      # Problme d'incompatibilit entre Py_UNICODE et Tcl_UniChar
      log.debug("%s: Erreur Unicode Python / UniChar Tcl" % self.name)
      text = item
      self.text.insert(start, text, itag)
    except TclError:
      log.error("%s: Erreur Tcl en essayant d'crire dans le pinni" % self.name)
      self.text.config(state='disabled')
      self.write(queueitem)
      return
    self.text.config(state='disabled')
    
    if start is None:
      log.error("%s: erreur d'index au niveau tags (ne devrait pas arriver)" % self.name)
      start = self.text.index("end-%s chars" % len(text))
    
    #== Ajout des tags
    pid = None
    for tag in tags.keys():
      if tag[:2] == "id":
        pid = tag
      for range in tags[tag]:
        self.text.tag_add(tag, "%s+%s chars" % (start, range[0]), "%s+%s chars" % (start, range[1]))
    
    #== Appel du processeur de posts pour les actions aprs insertion
    if action_type == PINNI_ADD:
      if pid is None:
        log.error("%s: Id de post non trouv dans les tags (ne devrait pas arriver)"% self.name)
      else:
        self.trib.pycc.processor.process_post(self.trib.bouchot, self.trib.posts[int(pid[2:])], 1)
    
    #== Scrolling vers le bas, si besoin
    if tobescrolled:
      self.text.see('end')
  
  def delete(self, start, end):
    log.debug("## Entering MutexWriter.delete")
    self.write((PINNI_DELETE, "", ("", start, end, "", 0)))
  
  def deleteFromTag(self, tag):
    log.debug("## Entering MutexWriter.deleteFromTag(%s)" % tag)
    self.write((PINNI_DELETE, tag, ("", -1, -1, "", 0)))
  
  def add(self, string, tags={}, tobescrolled=0):
    log.debug("## Entering MutexWriter.add(%s)" % string)
    self.write((PINNI_ADD, "", (string, -1, -1, tags, tobescrolled)))
  
  def insert(self, string, start, tags={}):
    log.debug("## Entering MutexWriter.insert(%s, %s)" % (string, start))
    self.write((PINNI_INSERT, "", (string, start, -1, tags, 0)))
  
  def insertFromTag(self, tag, string, tags={}):
    log.debug("## Entering MutexWriter.insertFromTag(%s, %s)" % (tag, string))
    self.write((PINNI_INSERT, tag, (string, -1, -1, tags, 0)))
  
  def replace(self, string, start, end, tags={}, tobescrolled=0):
    log.debug("## Entering MutexWriter.replace(%s, %s, %s)" % (string, start, end))
    self.write((PINNI_REPLACE, "", (string, start, end, tags, tobescrolled)))
  
  def replaceFromTag(self, tag, string, tags={}, tobescrolled=0):
    log.debug("## Entering MutexWriter.replaceFromTag(%s, %s)" % (tag, string))
    self.write((PINNI_REPLACE, tag, (string, -1, -1, tags, tobescrolled)))
  
  def setStyle(self, start, end, style):
    log.debug("## Entering MutexWriter.setStyle(%s, %s, %s)" % (start, end, style))
    self.write((PINNI_SETSTYLE, "", (style, start, end, "", 0)))
  
  def setStyleFromTag(self, tag, style):
    log.debug("## Entering MutexWriter.setStyleFromTag(%s, %s)" % (tag, style))
    self.write((PINNI_SETSTYLE, tag, (style, -1, -1, "", 0)))
  
  def replaceWithImage(self, tag, image):
    log.debug("## Entering MutexWriter.replaceWithImage(%s, imagedata)" % tag)
    imgtag = "img_%s" % tag
    self.write((PINNI_IMAGE, tag, (image, -1, -1, {imgtag: [(0,1)]}, 0)))
  

class Pinnipede(object):
  
  def __init__(self, master, tribune):
    log.debug("## Entering Pinnipede.__init__")
    self.trib = tribune
    self.root = master
    self._pinni = ScrolledText(master, wrap=WORD, state=DISABLED)
    self._pinni.see(END)
    self.pack = self._pinni.pack
    self.pack_forget = self._pinni.pack_forget
    
    self.images = {}
    
    self.search_mode = 0
    self.search_buf = ""
    self.search_len = IntVar()
    self.dummy_key = 0
    
    self.buf_ref = None
    self.buf_refidx = -1
    
    # Bindings d'vnements par dfaut
    # self._pinni.bind("<Enter>", lambda e, w=self._pinni: w.focus_set())
    self._pinni.bind("<Motion>", self.mouse_move)
    self._pinni.bind("<Control-Right>", self.showref_next)
    self._pinni.bind("<Control-Left>", self.showref_prev)
    self._pinni.bind("<Control-s>", self.searchnext)
    self._pinni.bind("<Escape>", self.cancel_modes)
    self._pinni.bind("<Alt-Down>", lambda e, w=self: w.trib.pycc.palmi.palmi.focus_set())
    self._pinni.bind("<KeyRelease>", self.keypressed)
    
    for i in range(1,4):
      self._pinni.bind("<ButtonRelease-%s>"%i, lambda e, w=self, act=i: w.mouse_click(e, act))
    
    # Emulation bouton du mileu
    if self.trib.pycc.options.val('emul_3but'):
      self._pinni.bind("<Control-ButtonRelease-1>", lambda e, w=self: w.mouse_click(e, 2))
    
    # Thread d'criture
    self._writer = MutexWriter(self)
    self._writer.start()
  
  def focus_set(self):
    self._pinni.focus_set()
  
  def clear(self):
    self._pinni.config(state='normal')
    self._pinni.delete('0.0', 'end')
    self._pinni.config(state='disabled')
  
  def cancel_modes(self, event=None):
    log.debug("## Entering Pinnipede.cancel_modes")
    if self.search_mode:
      self.search_mode = 0
      self.search_buf = ''
      self._pinni.tag_remove('hilifound', 1.0, END)
      self.trib.pycc.setStatus("Type ahed search mode [OFF]")
    self._pinni.tag_remove('newposts', 1.0, END)
  
  def kill(self):
    log.debug("## Entering Pinnipede.kill")
    self._writer.stop()
    self._pinni.destroy()
    self._pinni.vbar.destroy()
    self._pinni.frame.destroy()
  
  def addPost(self, p):
    log.debug("## Entering Pinnipede.addPost(%s) in %s" % (p.id, self.trib.name))
    tobescrolled = 1
    try:
      if self._pinni.dlineinfo("%s-1c" % END) is None:
        tobescrolled = 0
    except:
      # J'ai ajout un catch d'exception ici car Tkinter se vautre parfois sur la mthode dlineinfo
      # comme a, alatoirement, sans raison apparente (je sais, Tk sux)
      log.debug("%s: Tkinter s'est vautr sur dlineinfo" % self.trib.name)
    text, tags = self.trib.formatPost(p)
    self._writer.add(text, tags, tobescrolled)
  
  def replacePost(self, post, text, tags):
    log.debug("## Entering Pinnipede.replacePost(%s, %s)" % (post, text))
    pid = "id%s" % post.id
    self._writer.replaceFromTag(pid, text, tags)
  
  def delPost(self, pid):
    log.debug("## Entering Pinnipede.delPost(%s)" % pid)
    tag = "id%s"%pid
    self.deleteFromTag(tag)
  
  def deleteFromTag(self, tag):
    self._writer.deleteFromTag(tag)
  
  def insertPost(self, post, before_post):
    log.debug("## Entering Pinnipede.insertPost(%s, %s)" % (post, before_post))
    pid = "id%s" % before_post.id
    text, tags = self.trib.formatPost(post)
    self.insertFromTag(pid, text, tags)
  
  def insertFromTag(self, tag, text, tags):
    self._writer.insertFromTag(tag, text, tags)
  
  def showref_get(self):
    # print "---- Showref_get:", self.buf_ref, self.buf_refidx
    if self.trib.cur_post is None:
      self.trib.cur_post = self.trib.posts[self.trib.lastPostId]
      self.buf_refidx = -1
      self.buf_ref = None
    if self.buf_ref is None:
      self.buf_ref = [(r[0], t) for t, r in self.trib.cur_post.tags.items() if t[:3] == 'ref']
      self.buf_ref.sort()
      self.buf_refidx = -1
  
  def showref_prev(self, event=None):
    self.showref_get()
    if self.buf_refidx == -1:
      self.buf_refidx = len(self.buf_ref)
    self.buf_refidx -= 1
    # print "---- buf_ref, idx:", self.buf_ref, self.buf_refidx
    if self.buf_refidx < 0 or len(self.buf_ref) == 0:
      self.trib.cur_post = self.trib.cur_post.prev
      self.buf_ref = None
      self.showref_prev()
    self.showref_see()
  
  def showref_next(self, event=None):
    self.showref_get()
    self.buf_refidx += 1
    # print "---- buf_ref, idx:", self.buf_ref, self.buf_refidx
    if self.buf_refidx >= len(self.buf_ref):
      self.buf_refidx = -1
      self.trib.cur_post = self.trib.cur_post.next
      self.buf_ref = None
      self.showref_next()
    self.showref_see()
  
  def showref_see(self):
    self.clearRefs()
    post_y, post_e = self._pinni.tag_ranges("id%s" % self.trib.cur_post.id)[:2]
    self._pinni.mark_set(INSERT, post_y)
    self._pinni.see(self._pinni.index("%s-3lines" % post_y))
    #if self._pinni.dlineinfo(self._pinni.index("%s-3lines" % post_y)) is None:
    #  self._pinni.yview("scroll", -3, "units")
    self.showRef(self.buf_ref[self.buf_refidx][1], self.trib.cur_post)
  
  def showRef(self, genref, post, scroll=0):
    log.debug("## Entering Pinnipede.showRef(%s, %s, %s)" % (genref, post, scroll))
    ref, tribname = genref.split("@")
    tfound = 0
    for t in self.trib.getAllTribs():
      rtrib = self.trib.getTrib(t)
      if rtrib is None: continue
      if tribname.lower() in rtrib.getAliases():
        tfound = 1
        break
    if tfound:
      if not rtrib.loaded:
        self.trib.pycc.setStatus("Si la tribune '%s' tait active a irait mieux" % tribname)
        return
      if t == self.trib.bouchot.name:
        rpinni = None
        startpost = post
      else:
        if scroll:
          self.trib.pycc.select_tab(t)
          rpinni = None
        else:
          rpinni = self
        if not rtrib.lastPostId:
          self.trib.pycc.setStatus("La tribune '%s' est toute vide [:totoz]" % tribname)
          return
        startpost = rtrib.posts[rtrib.lastPostId]
      found_posts = rtrib.getPostsFromRef(ref, startpost)
      if len(found_posts) > 0:
        found_posts.sort()
        self.trib.pycc.popup.delete(0.0, END)
        for pid in found_posts:
          rtrib.pinni.highlight_post(pid, scroll, rpinni)
      else:
        if startpost.compare_time(ref) == -1:
          self.trib.pycc.setStatus("IPoT(tm) detected")
        else:
          self.trib.pycc.setStatus("O qu'il est, '%s' ?" % ref[3:])
    else:
      self.trib.pycc.setStatus("Gasp, la tribune '%s' est inconnue" % tribname)
  
  def clearRefs(self):
    log.debug("## Entering Pinnipede.clearRefs")
    self._pinni.tag_remove("hilightref", 1.0, END)
    self._pinni.tag_remove("urlhilight", 1.0, END)
    self._pinni.tag_remove("hilight", 1.0, END)
    if self.trib.pycc.popup_visible:
      self.trib.pycc.popup.place_forget()
      self.trib.pycc.popup_visible = 0
  
  def popup_post(self, pid, format_function=None, parent=None):
    log.debug("## Entering Pinnipede.popup_post(%s)" % pid)
    if format_function:
      p = self.trib.posts[pid]
      text, tags = format_function(p)
    else:
      start, stop = self._pinni.tag_ranges("id%s"%pid)[:2]
      text = self._pinni.get(start, stop)
    popup = self.trib.pycc.popup
    popup.mark_set(INSERT, END)
    index = popup.index(INSERT)
    popup.insert(index, text)
    if format_function:
      for tag in tags.keys():
        for range in tags[tag]:
          popup.tag_add(tag, "%s+%s chars" % (index, range[0]), "%s+%s chars" % (index, range[1]))
    else:
      cpt = 0
      while cpt < len(text):
        for tag in self._pinni.tag_names("%s+%s chars" % (start, cpt)):
          popup.tag_add(tag, "%s+%s chars" % (index, cpt))
        cpt += 1
    if not self.trib.pycc.popup_visible:
      if parent:
        apply_styles(popup, self.trib.bouchot.styles, 'popup', self.trib.pycc.default_styles)
      else:
        parent = self
      popup.place(in_=parent._pinni, relx=0, x=0, y=0, anchor=NW, relwidth=1)
      self.trib.pycc.popup_visible = 1
      h = 0
    else:
      h = int(popup.cget("height"))
    nh = len(popup.get(index, END)) / int(self._pinni.cget("width"))
    popup.config(height=int(nh)+h+1)
  
  def highlight_post(self, pid, scroll=0, rpinni=None):
    log.debug("## Entering Pinnipede.highlight_post(%s, %s)" % (pid, scroll))
    self.trib.cur_hipost = self.trib.posts[pid]
    nbref = self.underline_clocks(pid, rpinni)
    post_y, post_e = self._pinni.tag_ranges("id%s"%pid)[:2]
    self._pinni.tag_add("hilight", post_y, post_e)
    if self._pinni.dlineinfo(post_y) is None or self.trib.pycc.popup_visible or rpinni:
      if scroll:
        self._pinni.see(post_y)
        #self._pinni.yview("scroll", -1, "page")
        #self._pinni.yview("scroll", -1, "units")
        #if self._pinni.dlineinfo(post_e) is None:
        #  self._pinni.yview("scroll", 1, "units")
        self.trib.pycc.popup.place_forget()
        self.trib.pycc.popup_visible = 0
      else:
        self.popup_post(pid, parent=rpinni)
    if nbref > 1: sfx = "s"
    else: sfx = ""
    self.trib.pycc.setStatus("Horloge sur le post %s (%s rfrence%s)" % (pid, nbref, sfx))
  
  def setStyle(self, pid, style, target='id'):
    log.debug("## Entering Pinnipede.setStyle(%s, %s, %s)" % (pid, style, target))
    # beg, end = self._pinni.tag_ranges(target+str(pid))
    # self._pinni.tag_add(style, beg, end)
    self._writer.setStyleFromTag(target+str(pid), style)
  
  def remStyle(self, style):
    self._pinni.tag_remove(style, 1.0, END)
  
  def getSmiley(self, imgname):
    # print "-- getSmiley:", imgname
    if not self.images.has_key(imgname):
      try:
        imgurl = "http://%s/%s.gif" % (self.trib.options.val('smileys_server'), imgname)
        imgfile = self.trib.pycc.getImageWithCache(imgurl)
        self.images[imgname] = get_image(imgfile)
      except:
        # print "---- 'riv'po a afficher", imgname
        log.warning("%s: Pas moyen d'afficher '%s'" % (self.trib.name, imgname))
        return 0
    return 1
  
  def showSmiley(self, imgname):
    if not self.getSmiley(imgname): return
    cpt = len(self._pinni.tag_ranges("smiley_"+imgname))
    while cpt > 0:
      # print "---- replacing:", imgname
      # print "-- rangelist=", rangelist
      self._writer.replaceWithImage("smiley_"+imgname, self.images[imgname])
      cpt -= 2
  
  def underline_clocks(self, pid, rpinni=None):
    log.debug("## Entering Pinnipede.underline_clocks(%s)" % pid)
    # print "---- underline", pid
    res = 0
    if rpinni:
      pinni = rpinni._pinni
    else:
      pinni = self._pinni
    trib_aliases = self.trib.getAliases()
    tag_list = []
    for t in trib_aliases:
      tag_list += [ "ref"+ref+"@"+t for ref in self.trib.posts[pid].getAllRefs() ]
    # print "---- taglist", tag_list
    for tag in tag_list:
      tseq = pinni.tag_ranges(tag)
      # print "- ref", tag, "-range", tseq
      cpt = 0
      while cpt < len(tseq):
        pinni.tag_add("hilightref", tseq[cpt], tseq[cpt+1])
        cpt += 2
      res += cpt/2
    return res
  
  def setStyleTag(self, tag, style):
    log.debug("## Entering Pinnipede.setStyleTag(%s, %s)" % (tag, style))
    tseq = self._pinni.tag_ranges(tag)
    cpt = 0
    while cpt < len(tseq):
      self._pinni.tag_add(style, tseq[cpt], tseq[cpt+1])
      cpt += 2
  
  def get_tags_id(self, event):
    log.debug("## Entering Pinnipede.get_tags_id")
    tags = self._pinni.tag_names("@%d,%d" % (event.x, event.y))
    cur_id = -1
    tres = []
    for t in tags:
      if t[:2] == 'id':
        cur_id = int(t[2:])
      tres.append(t)
    tres.sort()
    tres.reverse()
    return cur_id, tres
  
  def searchnext(self, event=None):
    log.debug("## Entering Pinnipede.searchnext")
    self.dummy_key = 1
    if not self.search_mode:
      self.search_mode = 1
      self.trib.pycc.setStatus("Type ahed search mode [ON]")
      return
    # select = self._pinni.tag_ranges(SEL)
    select = self._pinni.tag_ranges('hilifound')
    if len(select):
      pos = select[1]
    else:
      pos = 1.0
    self.search(self.search_buf, pos)
  
  def search(self, text, startpos=1.0, case=0, regexp=0, stopindex=None):
    log.debug("## Entering Pinnipede.search(%s, %s)" % (text, startpos))
    self.trib.pycc.setStatus("Recherche: %s" % text)
    # self._pinni.tag_remove(SEL, 1.0, END)
    self._pinni.tag_remove('hilifound', 1.0, END)
    if not text:
      return None
    try:
      pos = self._pinni.search(text, startpos, nocase=not case, regexp=regexp, stopindex=stopindex, count=self.search_len)
    except:
      self.trib.pycc.setStatus("Expression invalide: '%s' " % text)
      return None
    if not pos:
      self.trib.pycc.setStatus("Recherche: %s (aucune occurence trouve)" % text)
      return None
    self._pinni.see(pos)
    # self._pinni.tag_add(SEL, pos, "%s+%sc" % (pos, len(text)))
    self._pinni.tag_add('hilifound', pos, "%s+%sc" % (pos, self.search_len.get()))
    return pos
  
  def keypressed(self, event):
    log.debug("## Entering Pinnipede.keypressed")
    k = str(event.keysym)
    # print "---- keypressed:", k
    if k == 'tab':
      self.trib.pycc.palmi.palmi.focus_set()
      return
    if not self.search_mode: return
    if self.dummy_key:
      self.dummy_key = 0
      return
    if k in ignorekeys: return
    if k[-2:] == "_R" or k[-2:] == "_L":
      k = k[:-2]
    if k in modifkeys: return
    if k in erasekeys:
      self.search_buf = self.search_buf[:-1]
    elif k == "space":
      self.search_buf += ' '
    else:
      self.search_buf += k
    # select = self._pinni.tag_ranges(SEL)
    select = self._pinni.tag_ranges('hilifound')
    if len(select):
      pos = select[0]
    else:
      pos = 1.0
    self.search(self.search_buf, pos)
  
  def mouse_move(self, event):
    self.focus_set()
    # Restauration des styles par dfaut
    self.trib.pycc.setStatus("")
    self._pinni.config(cursor="xterm")
    self.clearRefs()
    # Dtection de l'id du post point
    cur_id, tags = self.get_tags_id(event)
    if cur_id == -1: return
    self.trib.cur_post = self.trib.posts[cur_id]
    # Parcours des tags points
    #print "---", tags
    for t in tags:
      if t == 'time':
        self._pinni.config(cursor="arrow")
        self.highlight_post(cur_id)
        return
      elif t[:3] == "ref":
        self._pinni.config(cursor="arrow")
        self.showRef(t, self.trib.cur_post)
        self.trib.cur_ref = t
        return
      elif t == 'info' or t == 'login':
        self._pinni.config(cursor="arrow")
        self.trib.pycc.setStatus("Post id=%s UserAgent=%s " % (cur_id, self.trib.cur_post.info))
        return
      elif t[:7] == 'smiley_':
        # self._pinni.config(cursor="hand2")
        imgurl = "http://%s/%s.gif" % (self.trib.options.val('smileys_server'), t[7:])
        self.trib.pycc.setStatus("Smiley url: %s" % imgurl)
        return
      elif t[:4] == 'img_':
        self._pinni.config(cursor="arrow")
        self.trib.pycc.setStatus("Smiley [:%s]" % t[11:])
        return
      elif t[:4] == "url_":
        self.trib.pycc.setStatus(t[4:])
        self.setStyleTag(t, "urlhilight")
        self._pinni.config(cursor="hand2")
  
  def mouse_click(self, event, but=1):
    log.debug("## Entering Pinnipede.mouse_click(%s)" % but)
    self._pinni.tag_remove("newposts", 1.0, END)
    if but == 3:  # Menu popup
      self.trib.disp_ctxt_menu(event)
      return
    # Dtection de l'id du post point
    cur_id, tags = self.get_tags_id(event)
    if cur_id == -1: return
    self.trib.cur_post = self.trib.posts[cur_id]
    # Parcours des tags points
    tags.reverse()
    # print "---", tags
    for t in tags:
      if t == 'time':
        if but == 1:
          ref = self.trib.cur_post.dispTimeRef()
          if self.trib.getName() != self.trib.pycc.palmi.get_trib():
            ref += "@"+self.trib.getName()
          buf = ref+" "
        else: # but == 2
          buf = self.trib.cur_post.message
        self.trib.pycc.clipboard = buf
        try:
          self.trib.insertTextPalmi(buf)
        except SystemError:
          log.debug("%s: mouse_click: Erreur Unicode Python / UniChar Tcl" % self.trib.name)
          text = buf.decode(self.trib.options.val('encoding'))
          self.trib.pycc.clipboard = text
          self.trib.insertTextPalmi(text)
        # self.trib.pycc.palmi.palmi.focus_set()
        return
      elif t == 'login':
        if but == 1:
          self.trib.insertTextPalmi(self.trib.cur_post.login)
        else: # but == 2
          self.trib.pycc.openBrowser("http://%s/~%s/" % (self.trib.bouchot.options.val('http_server'), self.trib.cur_post.login))
        return
      elif t[:4] == 'img_':
        if but == 1:
          imgurl = "http://%s/%s.gif" % (self.trib.options.val('smileys_server'), t[11:])
          if self.trib.pycc.openBrowser(imgurl):
            self.trib.pycc.setUrlRead(imgurl)
          return
      elif t[:7] == "smiley_":
        imgurl = "http://%s/%s.gif" % (self.trib.options.val('smileys_server'), t[7:])
        if but == 1:
          if self.trib.pycc.openBrowser(imgurl):
            self.trib.pycc.setUrlRead(imgurl)
        else: # but == 2
          imgname = t[7:]
          if not self.getSmiley(imgname):
            self.trib.pycc.setStatus("Impossible d'afficher l'image %s" % imgname)
            return
          try:
            #popimg = Menu(self.trib.pycc.root, tearoff=0)
            #popimg.add("command", image=self.images[imgname], command=popimg.unpost)
            popimg = Label(self.root, image=self.images[imgname], border=2, relief='raised')
            popimg.bind("<ButtonRelease>", lambda e, p=popimg: p.destroy())
            popimg.place(in_=self._pinni, x=event.x, y=event.y)
            #popimg.post(event.x_root, event.y_root)
            #popimg.update_ideltasks()
          except:
            self.trib.pycc.setStatus("Impossible d'afficher l'image %s" % imgname)
        return
      elif t[:4] == "url_":
        url = t[4:]
        if but == 1:
          if self.trib.pycc.openBrowser(url):
            self.trib.pycc.setUrlRead(url)
        else: # but == 2
          self.trib.insertTextPalmi(url)
          self.trib.pycc.clipboard = url
      elif t[:3] == "ref":
        self.showRef(t, self.trib.cur_post, 1)
  
  def apply_styles(self, styles):
    log.debug("## Entering Pinnipede.apply_styles")
    apply_styles(self._pinni, styles, 'pinni', self.trib.pycc.default_styles)
  
