# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module Tribune
##   Dfinition du module de bouchot Tribune
##   (rcupration des posts, pinnipde, toussa)
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

from urllib import quote
from urllib2 import HTTPError, URLError
from htmlentitydefs import entitydefs

import sys, os.path, time
sys.path.append('../..')

from constants import *
from modules.module import *
from coinlib.persistants import *
from coinlib.baseclasses import PyccError, Post
from coinlib.parsers import *
from coinlib.threads import GenericThread
from coinlib.utils import open_url
from pinni import Pinnipede


def build_module(bouchot, name):
  return Tribune(bouchot, name)


class PostThread(GenericThread):
  """ Gestion du posting dans la tribune.
  """
  def __init__(self, name, tribune):
    log.debug("## Entering PostThread.__init__")
    self.trib = tribune
    GenericThread.__init__(self, name, type="queue")
  
  def process(self, args):
    log.debug("## Entering PostThread.process")
    message, auth, id, extra_arg = args
    self.trib.post_led.setState(STATE_HTTP)
    server = self.trib.bouchot.options.val('http_server')
    url = self.trib.options.val('add_post_url')
    ua = id.ua
    ua = ua.replace("%v", PYCC_VERSION)
    ua = ua.replace("%l", id.login)
    data = self.trib.options.val('add_post_data')
    data = data.replace("%m", quote(message))
    data = data.replace("%l", quote(id.login))
    tname = self.trib.getName()
    cookie = ''
    if auth and tname in id.cookies.keys():
      cookie = id.cookies[tname]
    result = None
    try:
      file = open_url(server, url, self.trib.pycc.options, ua, cookie, data)
      result = file.read()
      file.close()
    except HTTPError, value:
      if value.code == 302 or value.code == 304:
        log.debug("%s - %s" % (url, value))
      else:
        error = str(value)
        self.trib.post_led.setState(STATE_ERROR, error)
        log.error("%s: Erreur HTTP sur %s:\n    %s" % (self.name, url, error))
        return
    except URLError, value:
      error = str(value)
      self.trib.post_led.setState(STATE_ERROR, error)
      log.error("%s: Erreur en ouvrant l'url %s:\n    %s" % (self.name, url, error))
      return
    except:
      error = "Y'a eu un truc zarb sur %s" % url
      self.trib.post_led.setState(STATE_ERROR, error)
      log.error("%s: Erreur inconnue en ouvrant l'url %s" % (self.name, url))
      return
    if not result:
      result = "<ok>"
    else:
      result = result.strip()
    if result[0] != "<":
      self.trib.post_led.setState(STATE_WARNING, "Pas pu poster, rponse du serveur:\n%s" % result[:255])
      log.error("%s: Post a chou: %s" % (self.name, result))
      return
    self.trib.post_led.clear()
    self.trib.post_led.incr()
    log.debug("%s: Message ok" % self.name)
    self.trib.thread.interrupt()
  
  def write(self, message, auth, id):
    log.debug("## Entering PostThread.write(%s, %s, %s)" % (message, auth, id))
    self._thread.queue.put((message, auth, id, ""))
  

class TribuneOptions(OptionSet):
  """ Options du module Tribune
  """
  def setObjects(self):
    self._objects = {
      'descr':           Option("Description (nom pour affichage)", "Tribune"),
      'short_name':      Option("Nom court (pour la diode)", "T"),
      'backend':         Option("URL du fichier de backend", ""),
      'get_with_cookie': Option("Envoyer le(s) cookie(s) dans les requtes backend", 0, TYPE_BOOL),
      'docked':          Option("Pinnipde dock dans l'onglet", 1, TYPE_BOOL),
      'encoding':        Option("Encoding du backend", "iso-8859-15"),
      'check_delay':     Option("Nombre de secondes entre 2 checks de la tribune", 30, TYPE_INT),
      'add_post_url':    Option("URL pour poster dans la tribune", ""),
      'max_history':     Option("Nombre maximum de posts dans le pinnipde", 400, TYPE_INT),
      'wiki_url':        Option("Serveur Wiki",  "wiki.woof.lu/index.php"),
      'post_max_length': Option("Taille maximale des posts", 255, TYPE_INT),
      'slip_style':      Option("Type de slip", "Hybride", TYPE_CHOICE),
      'add_post_data':   Option("Donnes de post", ""),
      'smileys_server':  Option("Serveur de smileys", "forum.hardware.fr/images/perso"),
      'disp_smileys':    Option("Afficher les smileys directement dans le pinnipde", 0, TYPE_BOOL),
      'cookie_scheme':   Option("Schma cookie(s) d'authentification", ""),
      'disp_post_attrs': Option("Attributs de posts  afficher", ['time', 'login', 'message'], TYPE_CHOICE_MULTI),
      'toedit':          Option("Editables", { \
          "Gnral":   [ 'backend', 'add_post_url', 'add_post_data', 'get_with_cookie', 'slip_style', 'check_delay', 'post_max_length', 'wiki_url', 'smileys_server' ],
          "Affichage": [ 'max_history', 'disp_post_attrs', 'disp_smileys', 'encoding' ]
        } )
    }


class Tribune(Module):
  
  def init(self):
    log.debug("## Entering Tribune.init")
    self.posts = {}
    self.lastPostId = self.firstPostId = 0
    self.pycc = self.bouchot.pycc
    parent = self.bouchot.pw.add("%s:pinni" % self.name)
    self.pinni = Pinnipede(parent, self)
    self.pinni.pack(fill=BOTH, expand=1)
    self.pinni.apply_styles(self.bouchot.styles)
    self.viewer = self.pinni
    self.viewer_open = 1
    
    self.parser = PostParser()
    self.disp_post_attrs = self.options.val('disp_post_attrs')
    
    self.post_led = Led(self.bouchot.ledbar, "Post:")
    self.post_led.pack(side='right')
    for i in range(1,4):
      self.post_led._led.bind("<ButtonRelease-%s>"%i, lambda e, s=self, act=i: s.postled_click(act))
    if self.bouchot.pycc.options.val('emul_3but'):
      self.post_led._led.bind("<Control-ButtonRelease-1>", lambda e, s=self: s.postled_click(2))
    self.post_thread = PostThread("%s:post" % self.name, self)
    self.post_thread.start()
    
    self.bindKeys()
    self.bindClicks()
    
    self.cur_post = None    # Post actuellement point
    self.cur_hipost = None  # Post actuellement en surbrillance
  
  def get_http_params(self):
    cookie = ''
    if self.options.val('get_with_cookie'):
      idname = self.bouchot.options.val('default_id')
      id = self.bouchot.pycc.identities.get(idname)
      try:
        cookie = id.cookies[self.bouchot.name]
      except:
        pass
    return ('', cookie, None)
  
  def getAliases(self):
    log.debug("## Entering Tribune.getAliases")
    res = {}
    for n in self.bouchot.options.val('bouchot_aliases').split()+[self.bouchot.name]:
      res[n.lower()] = 1  # Astuce batarde pour s'assurer que l'on n'a pas de doublons
    return res.keys()
  
  def getAllTribs(self):
    log.debug("## Entering Tribune.getAllTribs")
    return self.pycc.bouchots.keys()
  
  def getTrib(self, name):
    log.debug("## Entering Tribune.getTrib")
    return self.pycc.bouchots[name].trib
  
  def getName(self):
    log.debug("## Entering Tribune.getName")
    return self.bouchot.name
  
  def setUrlRead(self, url):
    log.debug("## Entering Tribune.setUrlRead(%s)" % url)
    self.pinni.setStyleTag("url_"+url, "urlvisit")
  
  def getPostsFromPostRefs(self, post):
    plist = []
    for genref in [ t for t in post.tags.keys() if t[:3] == "ref" ]:
      ref, tribname = genref.split("@")
      tfound = 0
      for t in self.getAllTribs():
        rtrib = self.getTrib(t)
        if rtrib is None: continue
        if tribname.lower() in rtrib.getAliases():
          tfound = 1
          break
      if tfound:
        if not rtrib.loaded:
          continue
        if t == self.bouchot.name:
          startpost = post
        else:
          if not rtrib.lastPostId:
            continue
          startpost = rtrib.posts[rtrib.lastPostId]
        found_posts = rtrib.getPostsFromRef(ref, startpost)
        plist += [ (rtrib.bouchot, rtrib.posts[pid]) for pid in found_posts ]
    return plist
  
  def load_config(self):
    log.debug("## Entering Tribune.load_config")
    self.options = TribuneOptions(path=self.config_path)
  
  def unload(self):
    log.debug("## Entering Tribune.unload")
    self.post_thread.stop()
    self.post_led.destroy()
    self.pinni.kill()
  
  # On interdit l'ouverture / fermeture du pinnipde, soit la tribune
  # est charge et alors le pinni visible, soit tout est ferm
  def closeViewer(self):
    pass
  def openViewer(self):
    pass
  
  # Gestion des vnements clavier / souris
  def bindKeys(self):
    log.debug("## Entering Tribune.bindKeys")
    for keystroke, keyevent in self.pycc.pinni_keys.get().items():
      try:
        log.debug("%s: Configuring Key <%s> in pinni" % (self.name, keystroke))
        self.pinni._pinni.bind("<%s>" % keystroke, lambda e, ref=self, k=keyevent: ref.action_key(k))
      except:
        log.warning("%s: Impossible de configurer le raccourci clavier %s" % (self.name, keystroke))
  
  def unbindKeys(self):
    log.debug("## Entering Tribune.unbindKeys")
    for key in self.pycc.pinni_keys.get().keys():
      self.pinni._pinni.unbind(key)
  
  def bindClicks(self):
    log.debug("## Entering Tribune.bindClicks")
    for click, clickevent in self.pycc.pinni_clicks.get().items():
      try:
        self.pinni._pinni.bind("<%s>" % click, lambda e, ref=self, c=clickevent: ref.action_click(c, e))
      except:
        log.warning("%s: Impossible de configurer l'action souris " % (self.name, click))
  
  def unbindClicks(self):
    log.debug("## Entering Tribune.unbindClicks")
    for click in self.pycc.pinni_clicks.get().keys():
      self.pinni._pinni.unbind(click)
  
  def disp_ctxt_menu(self, event):
    log.debug("## Entering Tribune.disp_ctxt_menu")
    pid, context = self.pinni.get_tags_id(event)
    if pid == -1: return
    self.cur_post = self.posts[pid]
    self.pycc.ctxt_menu.delete(0, 100)
    self.pycc.ctxt_menu.add_command(label="Post: %s" % pid, command=self.nothing)
    self.pycc.ctxt_menu.add_separator()
    for entry, menuevent in self.pycc.pinni_menu.get().items():
      self.pycc.ctxt_menu.add_command(label=entry, \
        command=lambda s=self, m=menuevent, c=context, e=event: s.action_menu(m, c, e))
    self.pycc.ctxt_menu.post(event.x_root, event.y_root)
  
  def action_menu(self, menuevent, context, event):
    log.debug("%s: Action_menu: %s, %s" % (self.bouchot.name, menuevent, context))
    # menuevent.action.process(self.bouchot, PINNI, self.cur_post)
    self.action_event_process(menuevent, context, event, self.cur_post)
  
  def action_event_process(self, pyccevent, context, event, post):
    if pyccevent.action.type in actions_only_once:
      pyccevent.action.process(self.bouchot)
      return
    filter = keyword = None
    for tag in context:
      if tag in [ 'info', 'login', 'time', 'date' ]:
        filter = tag
        keyword = eval('post.'+filter)
        break
      elif tag[:3] == 'ref':
        idx_coord = "@%d,%d" % (event.x, event.y)
        filter = 'clock'
        start, end = self.pinni._pinni.tag_prevrange(tag, idx_coord)
        zeclock = self.pinni._pinni.get(start, end)
        keyword = tag
        break
      elif tag == 'message':
        idx_coord = "@%d,%d" % (event.x, event.y)
        filter = 'message'
        keyword = self.pinni._pinni.get(self.pinni._pinni.index(idx_coord+" wordstart"), self.pinni._pinni.index(idx_coord+" wordend"))
    if filter is None: return
    if filter == 'time':
      pyccevent.action.process(self.bouchot, PINNI, self.cur_post)
    else:
      tribcond = Condition('trib', OP_MATCHES, '(%s)' % self.bouchot.name)
      if filter == 'clock':
        ref, tribname = keyword.split("@")
        tfound = 0
        for t in self.getAllTribs():
          rtrib = self.getTrib(t)
          if rtrib is None: continue
          if tribname.lower() in rtrib.getAliases():
            tfound = 1
            break
        newcond = Condition('message', OP_MATCHES, zeclock)
        if tfound and rtrib.loaded:
          if t == self.bouchot.name:
            startpost = post
          elif rtrib.lastPostId:
            startpost = rtrib.posts[rtrib.lastPostId]
          else:
            startpost = None
          if startpost is not None:
            found_posts = rtrib.getPostsFromRef(ref, startpost)
            if len(found_posts) > 0:
              cset = ConditionSet([])
              for pid in found_posts:
                cset.add(Condition('id', OP_MATCHES, '(%s)'%pid))
              newcond = Condition('message', OP_HAS_REF_TO, cset=cset)
              # print "----", newcond, cset
      else:
        # Escapage des caractres spciaux pour regexps
        for car in regexp_escape:
          if car not in "[]^":
            keyword = keyword.replace(car, "["+car+"]")
          else:
            keyword = keyword.replace(car, "\\"+car)
        newcond  = Condition(filter, OP_MATCHES, '(%s)' % keyword)
      condset  = ConditionSet([tribcond, newcond], COND_ALL)
      # Processing sur les posts dj prsents
      for p in self.posts.values():
        if condset.match(self.bouchot, p):
          pyccevent.action.process(self.bouchot, PINNI, p)
      # Ajout au post-processor
      if pyccevent.addtopp:
        myfilter = None
        for filter in self.pycc.processor.get():
          if filter.name[:10] != "Pinnipede_":
            continue
          if filter.conditions == condset:
            myfilter = filter
            break
        if myfilter is None:
          newfilter = Filter("Pinnipede_%s" % md5.new(str(condset)).hexdigest(), [pyccevent.action], condset, 1, 1)
          self.pycc.processor.add_filter(newfilter)
        else:
          myfilter.add(pyccevent.action)
  
  def action_key(self, keyevent):
    log.debug("%s: Action_key: %s" % (self.bouchot.name, keyevent))
    if self.cur_hipost is None: return
    keyevent.action.process(self.bouchot, PINNI, self.cur_hipost)
  
  def action_click(self, clickevent, event):
    log.debug("%s: Action_click: %s, %s" % (self.bouchot.name, clickevent, event))
    pid, context = self.pinni.get_tags_id(event)
    if pid == -1: return
    self.cur_post = self.posts[pid]
    # clickevent.action.process(self.bouchot, PINNI, self.cur_post)
    self.action_event_process(clickevent, context, event, self.cur_post)
  
  def nothing(self, event=None):
    print "---- Tribune.Nothing called"
  
  def restaure_post(self, post):
    wiki = self.options.val('wiki_url')
    # old_slip = (self.options.val('slip_style') == "Hybride")
    post.message, post.tags = self.parser.parsePost(post.original_text, self.bouchot.name, wiki) #, old_slip)
    post.status = []
    text, tags = self.formatPost(post)
    self.replaceFromTag("id%s" % post.id, text, tags)
  
  def replaceFromTag(self, tag, newtext, newtags):
    self.pinni._writer.replaceFromTag(tag, newtext, newtags)
  
  def getPostsFromRef(self, ref, p):
    log.debug("## Entering Tribune.getPostsFromRef(%s, %s)" % (ref, p))
    found_posts = []
    res_comp = 1
    post_cpt = p
    while post_cpt.id >= self.firstPostId:
      res_comp = post_cpt.compare_time(ref)
      if res_comp == 0: found_posts.append(post_cpt.id)
      post_cpt = post_cpt.prev
      if post_cpt is None: break
    return found_posts
  
  def insertTextPalmi(self, text, pos=None, del_selection=0):
    log.debug("## Entering Tribune.insertTextPalmi(%s, %s, %s)" % (text, pos, del_selection))
    self.pycc.palmi.palmi.inserter(text, pos, del_selection)
    self.pycc.palmi.tabselect('Message')
    self.pycc.palmi.palmi.focus_set()
  
  def formatPost(self, p, attrs=None):
    log.debug("## Entering Tribune.formatPost(%s, %s)" % (p, attrs))
    tags = {}
    text = ur""
    start = 0
    if attrs is None:
      attrs = self.disp_post_attrs # self.options.val('disp_post_attrs')
    for attr in attrs:
      buf = p.disp(attr)
      if buf == ur"" and attr == 'login':
        buf = p.disp('info')
        attr = 'info'
      elif attr == 'id':
        buf = u"[%-6s]" % buf
        attr = 'pid'
      end = start + len(buf)
      tags[attr] = [ (start, end) ]
      tags[attr+p.disp('id')] = [ (start, end) ]
      if attr == 'message':
        for t in p.tags.keys():
          tags[t] = [(start+x, start+y) for x, y in p.tags[t]]
      if len(buf) > 0:
        text += buf+ur" "
        start = end+1
    text = text+"\n"
    tags['id'+p.disp('id')] = [ (0, len(text)) ]
    for tag in [toUnicode(t) for t in tags.keys()]:
      if tag[4:] in self.pycc.urlread:
        try:
          tags['urlvisit'] += tags[tag]
        except KeyError:
          tags['urlvisit'] = tags[tag]
    return text, tags
  
  def apply_styles(self):
    self.pinni.apply_styles(self.bouchot.styles)
  
  def apply_config(self):
    newattrs = self.options.val('disp_post_attrs')
    if "".join(self.disp_post_attrs) != "".join(newattrs):
      self.disp_post_attrs = newattrs
      self.refresh_all()
  
  def parse(self, file):
    log.debug("## Entering Tribune.parse")
    tmplist = []
    line = file.readline()
    if line[:6] == "Error:":
      self.error = line
      file.close()
      log.warning("%s: le serveur a dit: %s" % line)
      return
    encoding = self.getEncoding(line)
    wiki = self.options.val('wiki_url')
    slip_style = self.options.val('slip_style')
    while 1:
      if not line or line.find('</board>') >=0: break
      start = line.find('<post')
      (info, login, message) = ("","","")
      if start >= 0:
        try:
          id = getAttr("id", line)
          if self.lastPostId >= int(id): break
          time = getAttr("time", line)
          while line.find("</post>") < 0:
            line = file.readline()
            if not line: break
            line = line.strip()
            if line.find("<info") > -1:
              if line.find("/>") < 0:
                while line.find("</info>") < 0:
                  str = file.readline()
                  if not str: break
                  line += str.strip()
                info = zapEntities(getContent("info", line))
            if line.find("<message") > -1:
              if line.find("message/>") < 0:
                while line.find("</message>") < 0:
                  str = file.readline()
                  if not str: break
                  line += str
                message = zapComments(getContent("message", line))
            if line.find("<login") > -1:
              if line.find("/>") < 0:
                while line.find("</login>") < 0:
                  str = file.readline()
                  if not str: break
                  line += str.strip()
                login = zapEntities(getContent("login", line))
          orig_txt = self.parser.htmlParser.pre_parse(message, slip_style, encoding)
          msg, tags = self.parser.parsePost(orig_txt, self.bouchot.name, wiki)
        except MiniXmlError, e:
          log.error("%s: Erreur MiniXML: %s" % (self.name, e))
          self.error = str(e)
          break
        try:
          login = login.decode(encoding)
        except:
          pass
        try:
          info = info.decode(encoding)
        except:
          pass
        p = Post(id, time, login, info, msg, tags)
        p.original_text = orig_txt
        tmplist.append(p)
      else:
        line = getLine(file)
    return tmplist
  
  def process(self, posts=[]):
    log.debug("## Entering Tribune.process")
    if not posts: return
    posts.sort()
    for p in posts:
      if self.firstPostId == 0:
        self.firstPostId = p.id
      if self.lastPostId > 0:
        p.prev = self.posts[self.lastPostId]
        p.prev.next = p
        if p.same_time(p.prev):
          if p.prev.num == 0: p.prev.set('num', 1)
          p.set('num', p.prev.num+1)
      self.posts[p.id] = p
      self.lastPostId = p.id
      self.pycc.processor.pre_process(self.bouchot, p)
      # self.pinni.addPost(p)
    log.info("%s: %s nouveaux posts - dernier id: %s" % (self.name, len(posts), self.lastPostId))
    self.bouchot.setState(STATE_NEW)
    while len(self.posts) > self.options.val('max_history'):
      tobepurged = self.firstPostId
      log.info("%s: Trop de posts, je purge %s" % (self.name, tobepurged))
      try:
        self.pinni.delPost(tobepurged)
      except:
        self.warning("%s: Impossible d'effacer le post %s du pinnipde" % (self.name, tobepurged))
      else:
        self.firstPostId = self.posts[tobepurged].next.id
        del self.posts[tobepurged]
        self.posts[self.firstPostId].prev = None
    self.led.incr(len(posts))
  
  def poster(self, msg, auth=1, id=None):
    log.debug("## Entering Tribune.poster(%s, %s, %s)" % (msg, auth, id))
    if id is None:
      id = self.pycc.identities.get(self.bouchot.options.val('default_id'))
    myhash = { '%u': id.ua, '%l': id.login, '%v': PYCC_VERSION, '%h': time.strftime("%H:%M:%S", time.localtime()) }
    txt = escape_spec_chars(msg, myhash)
    try:
      txt = txt.encode(self.options.val("encoding"))
      # print "------- message  poster encod"
    except:
      log.warning("%s: beuh, je sais pas encoder le message en %s" % (self.name, self.options.val("encoding")))
    self.post_thread.write(txt, auth, id)
  
  def postled_click(self, action):
    log.debug("## Entering Tribune.postled_click(%s)" % action)
    if action == 1:
      log.info("%s: click diode post, no action" % self.name) # AFR ?
    elif action == 2:
      self.post_led.showMsg(self.bouchot.pw.interior(), "Dernire erreur sur %s" % self.name, SE)
    else:
      self.post_thread.restart()
  
  def refresh_all(self):
    self.pinni.clear()
    slist = [pid for pid in self.posts.keys()]
    slist.sort()
    for pid in slist:
      self.pinni.addPost(self.posts[pid])
    # for pid in self.posts.keys():
      # self.refresh_post(self.posts[pid])
  
  def refresh_post(self, post):
    self.pycc.processor.process_post(self.bouchot, post, 0)
    self.restaure_post(post)
    self.pycc.processor.process_post(self.bouchot, post, 1)
  