# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module userinfo
##   Dfinition du module de bouchot UserInfo :
##   monitoring des infos sur l'utilisateur (XPs, votes...)
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')

import sys, os, os.path, md5
sys.path.append('../..')

from constants import *
from modules.module import *
from coinlib.persistants import OptionSet, Option
#from coinlib.parsers import handle_unicode
from coinlib.utils import *

def build_module(parent, name):
  return UserInfo(parent, name)


class UserInfoOptions(OptionSet):
  """ Options du module UserInfo
  """
  def setObjects(self):
    self._objects = {
      'descr':        Option("Description (nom pour affichage)", "User Info & Messagerie"),
      'short_name':   Option("Nom court (pour la diode)", ""),
      'backend':      Option("URL du fichier de backend", ""),
      'docked':       Option("Dock dans l'onglet", 1, TYPE_BOOL),
      'check_delay':  Option("Nombre de secondes entre 2 checks du backend", 300, TYPE_INT),
      'srv_type':     Option("Type de serveur", "templeet", TYPE_CHOICE),
      'max_history':  Option("Nombre maximum d'lments  conserver", 50, TYPE_INT),
      'toedit':       Option("Editables", { \
          "Gnral": [ 'descr', 'backend', 'check_delay', 'srv_type', 'max_history', 'docked' ]
        } )
    }

class MsgItem(object):
  
  def __init__(self, attrs={}):
    self.attributes = attrs
    if attrs.has_key('id'):
      self.id = attrs['id']
    else:
      self.computeId()
    self.read = 0     # Flag lu/nonlu
    self.next = None  # Pointeur sur le suivant
  
  def computeId(self):
    self.id = md5.new(str(self.attributes)).hexdigest()
  
  def __str__(self):
    return repr(self.attributes['title'])
  
  def getTitle(self):
    return self.attributes['title']
  
  def getStatus(self):
    return self.attributes['status']
  
  def getLink(self):
    return self.attributes['link']
  
  def getDate(self):
    try:
      return self.attributes['date']
    except:
      return ""
  
  def getAuthor(self):
    return self.attributes['author']
  
  def markRead(self):
    self.read = 1
  
  def markUnread(self):
    self.read = 0
  

class MsgViewer(Frame):

  def __init__(self, parent, userinfo):
    self.root = parent
    self.userinfo = userinfo
    self.styles = self.userinfo.bouchot.styles
    Frame.__init__(self, parent)
    self.build_interface()
    self.update()
  
  def build_interface(self):
    fr = Frame(self)
    self.butmenu = Label(fr, text="Actions >", relief=FLAT, border=2)
    self.butmenu.bind("<Enter>", lambda e, s=self: s.butmenu.config(relief=RAISED))
    self.butmenu.bind("<Leave>", lambda e, s=self: s.butmenu.config(relief=FLAT))
    self.butmenu.bind("<1>", self.show_menu)
    self.butmenu.pack(side=LEFT)
    self.butclose = Label(fr, text=" X ", relief=FLAT, border=2)
    self.butclose.bind("<Enter>", lambda e, s=self: s.butclose.config(relief=RAISED))
    self.butclose.bind("<Leave>", lambda e, s=self: s.butclose.config(relief=FLAT))
    self.butclose.bind("<1>", self.quit)
    self.butclose.pack(side=RIGHT)
    self.label = Label(fr, font="Helvetica 10 bold", background=COLOR_TITLE)
    self.label.pack(side=LEFT, fill=X, expand=1)
    fr.pack(fill=X)
    self.menu = Menu(self.root, tearoff=0)
    self.menu.add_command(label="Ouvrir les non lus", command=self.openAll)
    self.ascending = IntVar()
    self.ascending.set(1)
    self.menu.add_checkbutton(label="Les plus rcents en haut", command=self.toggleSort, variable=self.ascending)
    self.menu.bind("<Escape>", self.hide_menu)
    if self.userinfo.options.val('docked'):
      btxt = "Dtacher"
    else:
      btxt = "Docker"
    self.menu.add_command(label=btxt, command=self.userinfo.toggleViewMode)
    self.menu.add_command(label="Configurer module...", command=self.showConfig)
    self.menu.add_command(label="Fermer", command=self.quit)
    self.list = ScrolledText(self, wrap=WORD)
    self.list.pack(side=RIGHT, fill=BOTH, expand=1)
    self.list.bind("<Enter>", lambda e, w=self: w.list.focus_set())
    self.list.bind("<Motion>", self.mouse_move)
    self.list.bind("<1>", self.mouse_click)
    self.list.bind("<3>", self.show_menu)
    self.list.bind("<Escape>", self.hide_menu)
  
  def show_menu(self, event):
    self.menu.post(event.x_root, event.y_root)
  
  def hide_menu(self, event=None, **kw):
    self.menu.unpost()
  
  def showConfig(self, event=None):
    self.userinfo.bouchot.pycc.showConfig(self.userinfo.name.replace(":", "_"))
  
  def update(self):
    """ Mise  jour du contenu de la fentre """
    self.setLbl(self.userinfo.title)
    self.list.config(state=NORMAL)
    self.list.config(cursor="arrow")
    # Effacement de l'ancien contenu
    self.list.delete(0.0, END)
    if len(self.userinfo.objects.keys()) > 0:
      # Configuration des styles
      self.apply_styles()
      item = self.userinfo.objects[self.userinfo.firstId]
      # Insertion des items
      while item is not None:
        self.insertItem(item)
        item = item.next
    else:
      self.list.insert(END, "Aucun lment trouv !")
    self.list.config(state=DISABLED)
  
  def apply_styles(self):
    apply_styles(self.list, self.styles, 'pinni', self.userinfo.bouchot.pycc.default_styles)
    self.list.tag_config('descr', lmargin1=20, lmargin2=20, borderwidth=1, relief=FLAT)
  
  def toggleSort(self, event=None):
    cursort = self.ascending.get()
    self.update()
  
  def mouse_move(self, event):
    self.unhilight()
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      if t[:4] == "url_":
        self.userinfo.bouchot.pycc.setStatus(t[4:])
        self.setStyleTag(t, "urlhilight")
        self.list.config(cursor="hand2")
  
  def mouse_click(self, event):
    tags = self.list.tag_names("@%d,%d" % (event.x, event.y))
    for t in tags:
      if t[:4] == "url_":
        url = t[4:]
        self.userinfo.bouchot.pycc.openBrowser(url)
        self.setStyleTag(t, "urlvisit")
        self.userinfo.bouchot.pycc.setUrlRead(url)
      if t[:2] == "id":
        self.userinfo.thread.restart()
    self.hide_menu()
  
  def setStyleTag(self, tag, style):
    tseq = self.list.tag_ranges(tag)
    cpt = 0
    while cpt < len(tseq):
      self.list.tag_add(style, tseq[cpt], tseq[cpt+1])
      cpt += 2
  
  def hilight(self, id):
    """ Mise en surbrillance d'un lien """
    beg, end = self.list.tag_ranges("id%s" % id)
    self.list.tag_add("urlhilight", beg, end)
    self.list.config(cursor="hand2")
  
  def unhilight(self, event=None):
    """ Enlve les surbrillances """
    self.list.tag_remove("urlhilight", 1.0, END)
    self.list.config(cursor="arrow")
    self.userinfo.bouchot.pycc.setStatus("")
  
  def openAll(self, event=None):
    for item in [i for i in self.userinfo.objects.values() if not i.read]:
      self.openItem(item)
  
  def openItem(self, item):
    """ Ouvre un lien dans un browser """
    self.userinfo.bouchot.pycc.openBrowser(item.getLink())
    # self.userinfo.markRead(item)
  
  def insertItem(self, item):
    self.list.config(state=NORMAL)
    if self.ascending.get():
      self.list.mark_set(INSERT, 0.0)
    else:
      self.list.mark_set(INSERT, END)
    self.list.insert(INSERT, item.getDate(), "time")
    self.list.insert(INSERT, " [%s] " % item.getStatus())
    self.list.insert(INSERT, item.getAuthor(), "login")
    self.list.insert(INSERT, " ")
    t = "id%s" % item.id
    self.list.insert(INSERT, item.getTitle(), t)
    self.list.insert(INSERT, "\n")
    self.setStyleTag(t, "a")
    self.setStyleTag(t, "url_"+item.getLink())
    self.list.config(state=DISABLED)
    if item.read:
      self.markRead(item)
  
  def setLbl(self, text):
    self.label.config(text=text)
  
  def markRead(self, item):
    # Marque un lien comme lu
    self.setStyleTag("url_"+item.getLink(), "urlvisit")
  
  def quit(self, event=None):
    self.userinfo.closeViewer()



class UserInfo(Module):
  
  def init(self):
    self.lastId = None
    self.firstId = None
    self.title = "Messagerie DLFP"
  
  def build_viewer(self, parent):
    self.viewer = MsgViewer(parent, self)
  
  def apply_styles(self):
    self.viewer.apply_styles()
  
  def load_config(self):
    self.options = UserInfoOptions(path=self.config_path)
  
  def unload(self):
    # AFR ?
    pass
  
  def get_http_params(self):
    idname = self.bouchot.options.val('default_id')
    id = self.bouchot.pycc.identities.get(idname)
    try:
      cookie = id.cookies[self.bouchot.name]
    except:
      cookie = ''
    return ('', cookie, None)
  
  def parse(self, file):
    xp = ""
    text = toUnicode(file.read())
    start = end = text.find("Avis :")
    if start >= 0:
      start += 7
      end = text.find("/", start)
      reste = text[start:end]
      start = end+1
      end = text.find("\n", start)
      votes = text[start:end]
    else:
      votes = reste = "?"
      end = 0
    tmplist = []
    start = text.find('<input type="checkbox" name="table[', end)
    while start > end:
      attrs = {}
      end = text.find('<td>', start)
      if end < start: break
      start = text.find('</td>', end)
      if start < end: break
      attrs['status'] = text[end+4:start].replace("\n", " ").strip()
      start = text.find('<a href="../~', end)
      if start < end: break
      end = text.find('/">', start)
      if end < start: break
      pseudo = text[start+13:end]
      start = text.find('</a>', end)
      if start < end: break
      attrs['author'] = text[end+4:start].replace("\n", " ").strip() + " (%s)" % pseudo
      end = text.find('<a href="viewmessage,', start)
      if end < start: break
      start = text.find('.html">', end)
      if start < end: break
      attrs['id'] = text[end+21:start]
      end = text.find('</a>', start)
      if end < start: break
      attrs['title'] = text[start+7:end].replace("\n", " ").strip()
      attrs['link'] = "http://%s%sviewmessage,%s.html" % (self.bouchot.options.val('http_server'), self.options.val('backend'), attrs['id'])
      newitem = MsgItem(attrs)
      newitem.read = self.isRead(newitem)
      tmplist.append(newitem)
      start = text.find('<input type="checkbox" name="table[', end)
    tmplist.reverse()
    return (xp, reste, votes), tmplist
  
  def process(self, results):
    infos, items = results
    txt = "%sVotes: %s/%s" % infos
    self.led.setLabel(txt)
    oldids = [id for id in self.objects.keys()]
    self.objects = {}
    self.lastId = None
    self.firstId = None
    for msgitem in items:
      if self.firstId is None:
        self.firstId = msgitem.id
      if msgitem.id not in oldids:
        log.debug("%s: nouveau msgitem: %s" % (self.name, msgitem.id))
        self.bouchot.setState(STATE_NEW)
      if self.lastId is not None:
        self.objects[self.lastId].next = msgitem
      self.lastId = msgitem.id
      self.objects[msgitem.id] = msgitem
    if self.viewer_open:
      self.viewer.update()
    self.updateLed()
  
  def updateLed(self):
    self.led.setText(str(len([i for i in self.objects.values() if not i.read])))
  
  def markRead(self, item):
    item.read = 1
    if self.viewer_open:
      self.viewer.markRead(item)
    self.updateLed()
  
  def isRead(self, item):
    return item.attributes['status'] == "lu"
