# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Plug-in Plateau de Fruits de Mer
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *

from time import *
import os.path

def build_plugins(pycc):
  return { \
    'pdfm': Pdfm(pycc, "Plateau de fruits de mer", "Sauvegarde la tribune courante dans un fichier HTML")
  }

class Pdfm(Plugin):
  
  def action_menu(self):
    self.process(self.pycc.bouchots[self.pycc.get_bouchot()])
  
  def process(self, bouchot, context=PINNI, post=None):
    if bouchot.trib is None: return
    if not bouchot.trib.loaded: return
    tstamp = gmtime(time())
    datetime = strftime("%d/%m/%Y  %H:%M:%S")
    shots_path = os.path.join(bouchot.pycc.config_path, "shots")
    if not os.path.exists(shots_path):
      os.mkdir(shots_path)
    pdfm_file = os.path.join(shots_path, strftime(bouchot.name+"_%Y-%m-%d-%H%M%S.html", tstamp))
    title = "Tribune %s - Plateau de fruits de mer" % (bouchot.name)
    
    html_head = """<?xml version="1.0" encoding="iso-8859-15"?>
  <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "DTD/xhtml1-strict.dtd">
  <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="fr" lang="fr">
  
  <head>
    <title>%s</title>
  </head>
    
  <body>
    <style type="text/css">
    BODY { font-family: Arial, Helvetica, sans-serif; font-size: 9pt; }
    .clock { color: #004000; }
    .login { color: #800000; }
    .ua { color: #404000; }
    .refclock { color: #202080; }
    </style>
    <script language="javascript" type="text/javascript">
      function showref(id) {
        post = document.getElementById(id);
        post.obg = post.style.background;
        post.style.background = "#FCFCDA";
      }
      function clearref(id) {
        post = document.getElementById(id);
        post.style.background = post.obg;
      }
    </script>
    <h1>%s</h1>
    <table cellspacing="1" cellpadding="1">
  """ % (title, title)
  
    html_foot = """  </table>
    <hr />
    <p style="text-align:center;font-size:8pt">Gnr le %s</p>
  </body>
  </html>""" % (datetime)
  
    html_post = """    <tr id="%s" style="background-color:%s">
        <td class="clock">%s</td>
        <td class="login">%s</td>
        <td class="ua">%s</td>
        <td class="message">%s</td>
      </tr>
  """
  
    f = open(pdfm_file, "w")
    f.write(html_head)
    
    # On recre la liste des posts car si on utilisait directement trib.posts, le tableau
    # risque de changer pendant la boucle for (vu qu'a priori le thread de polling de
    # la tribune continue de tourner pendant l'excution du plugin)
    postlist = [post for id, post in bouchot.trib.posts.items()]
    postlist.sort()
    
    cpt_post = 0
    for post in postlist:
      message = ""
      cpt = 0
      if cpt_post %2:
        color = "#FFFFFF"
      else:
        color = "#F0F0FC"
      for ch in post.message:
        for tag in ['a', 'b', 's', 'u', 'i', 'tt', 'clock']:
          if tag in post.tags:
            for begin, end in post.tags[tag]:
              if begin == cpt:
                if tag == 'clock':
                  message += '<span class="refclock"'
                else:
                  message += "<"+tag
                if tag == "a":
                  for urltag in post.tags.keys():
                    if urltag[:4] == "url_":
                      for mark in post.tags[urltag]:
                        if mark[0] == begin:
                          message += ' href="%s"' % urltag[4:]
                if tag == "clock":
                  for reftag in post.tags.keys():
                    if reftag not in ['a', 'b', 's', 'u', 'i', 'tt', 'clock'] and reftag[:4] != "url_":
                      for mark in post.tags[reftag]:
                        if mark[0] == begin:
                          message += ' onmouseover="showref(\'%s\')" onmouseout="clearref(\'%s\')"' % (reftag[3:9], reftag[3:9])
                message += ">"
        for tag in ['clock', 'tt', 'i', 'u', 's', 'b', 'a']:
          if tag in post.tags:
            for begin, end in post.tags[tag]:
              if end == cpt:
                if tag == 'clock':
                  message += "</span>"
                else:
                  message += "</%s>" % tag
        message += ch
        cpt += 1
      cpt_post += 1
      f.write(html_post % ("".join(post.disp('time').split(":")), color, post.disp('time'), post.login, post.info, message))
    
    f.write(html_foot)
    f.close()
    
    bouchot.pycc.setStatus("Plateau de Fruits de Mer sauvegard dans %s" % pdfm_file)
  
