# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Module Pendu
##   Ce module dfinit le plug-in Pendu - un bot pour jouer au pendu.
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')

from constants import *
from coinlib.plugin import *
from coinlib.persistants import Identity, Option, OptionSet, Condition, ConditionSet, Filter, ActionPlugin
from cointklib.dialogs import CoinDialog
from cointklib.basicwidgets import *

import os, os.path, re
import tkMessageBox
# from Tkinter import Label
from random import randrange, choice

def build_plugins(pycc):
  return { \
    'pendu': Pendu(pycc, "Pendu", "Un bot pour jouer au pendu")
  }

ident = Identity(login="Pendu", ua="Pendu", cookies={'koocotte':'user=Pendu'})

class PenduServer(object):
  
  def __init__(self, dico_path, trib):
    self.trib = trib
    self.dico_path = dico_path
    self.re_letter = re.compile("^.*[<]\s*([A-Za-z]+)\s*.*$")
    self.hang_steps = [ "", "O", "|", ".-", "-.", "/", "\\", "RIP", "Gagn!" ]
    self.server_running = 0
    self.game_running = 0
    self.words = []
    # self.load_words(self.dico_path)
  
  def load_words(self, dico_path):
    try:
      f = open(dico_path, "r")
      self.words = [ word[:-1] for word in f.readlines() ]
      f.close()
      self.dico_path = dico_path
      log.debug("### PENDU dictionnaire recharg")
    except:
      tkMessageBox.showwarning("Plugin Pendu", "Impossible de charger le fichier de mots\n%s.\n Vrifiez le chemin." % dico_path)
  
  def start_game(self):
    if self.game_running:
      return # already running
    if len(self.words) == 0:
      log.debug("### PENDU aucun mot dans le dico !")
      return
    log.info("### PENDU new game")
    # Initialisation
    self.hidden_word = choice(self.words).lower()  # Choix du mot
    log.debug("### PENDU:", self.hidden_word)
    self.letters = []     # Liste des lettres joues
    self.hang_level = 0   # Niveau de pendaison
    self.game_running = 1 # Boolen partie en cours
  
  def play(self, post):
    if not self.game_running:
      self.start_game()
    # Vrification de la validit du coup jou
    pendu_invoke = self.re_letter.match(post.message)
    if pendu_invoke:
      played_letters = pendu_invoke.groups()[0].lower()
    else:
      played_letters = ""
    if len(played_letters) > len(self.hidden_word):
      played_letters = ""
    # Vrification des lettres nouvellement joues/dj joues
    valid_play = 0
    for letter in played_letters:
      if letter not in self.letters:
        valid_play = 1
        self.letters.append(letter)
        self.letters.sort()
        if letter not in self.hidden_word:
          self.hang_level += 1  # Mauvaise lettre
    if not valid_play and len(self.letters) > 0:
      log.debug("### PENDU not valid turn")
      return
    if self.hang_level >= 7:
      # Partie perdue
      self.trib.poster(u"%s [Pendu] <b>Perdu!</b> C'tait: <tt>%s</tt>" % (post.dispTimeRef(), self.hidden_word), 1, ident)
      self.game_running = 0
      return
    # Construction du mot avec les lettres trouves
    resword = ""
    for c in self.hidden_word:
      if c in self.letters:
        resword += c
      else:
        resword += '-'
    if '-' not in resword:
      # Partie gagne
      self.hang_level = -1
      self.game_running = 0
      self.hang_steps[self.hang_level] = u"%s a gagn!" % post.login 
    # On poste le rsultat
    if len(self.letters) > 0:
      score = "%2.2f%%" % ((100.0*len([l for l in self.letters if l in self.hidden_word]))/len(self.letters))
    else:
      score = ""
    self.trib.poster(u"%s [Pendu] <b>%s</b> <tt> %s </tt> <s>%s</s>  <i>%s</i>" % (post.dispTimeRef(), self.hang_steps[self.hang_level], resword, "".join([l for l in self.letters if l not in self.hidden_word]), score), 1, ident)

class PenduOptions(CoinDialog):
  
  def __init__(self, plug, pendu, parent, title, buttons):
    self.plug = plug
    self.pendu = pendu
    CoinDialog.__init__(self, parent, title, buttons)
  
  def body(self, master):
    if self.pendu.server_running:
      Label(master, text="Bouchot %s: Pendu Server actif !" % self.pendu.trib.getName(), background='#80FF80').pack(pady=2, fill=X)
    else:
      Label(master, text="Bouchot %s: aucun Pendu Server dmarr." % self.pendu.trib.getName(), background='#FF8080').pack(pady=2, fill=X)
    self.wdico = FileField(master, "Dictionnaire", value=self.pendu.dico_path)
    self.wdico.pack(pady=2, fill=X)
  
  def bind_button(self, button):
    if button == 'Lancer':
      return self.closeStart
    else:
      return self.closeStop
  
  def closeStart(self):
    #self.pendu.dico_path = self.wdico.get()
    self.pendu.load_words(self.wdico.get())
    self.plug.res = 1
    self.cancel()
  
  def closeStop(self):
    self.plug.res = 0
    self.cancel()
  
class Pendu(Plugin):
  
  def init(self):
    self.servers = {}
  
  def action_menu(self):
    cur_trib = self.pycc.bouchots[self.pycc.get_bouchot()].trib
    if cur_trib is None or not cur_trib.loaded: return
    
    if self.servers.has_key(cur_trib.getName()):
      pendu = self.servers[cur_trib.getName()]
    else:
      pendu = PenduServer(os.path.join('plugins', 'pendu.words'), cur_trib)
      self.servers[cur_trib.getName()] = pendu
    
    # boite de dialogue dico + bouton dmarrer/stopper serveur + bouton recharger dico
    #res = dialog toto...
    self.res = -1
    PenduOptions(self, pendu, self.pycc.root, "Serveur de jeu Pendu", ["Lancer", "Arrter", BUT_CANCEL])
    
    myfilter = None
    pos = 0
    for filter in self.pycc.processor.get():
      if filter.name == "Pendu_server":
        myfilter = filter
        break
      pos += 1
    if self.res == 1:    # start server
      if myfilter is None:
        tribcond = Condition('trib', OP_MATCHES, '(%s)' % cur_trib.getName())
        penducond = Condition('message', OP_MATCHES, '[Pp][Ee][Nn][Dd][Uu][<]')
        condset  = ConditionSet([tribcond, penducond], COND_ALL)
        action = ActionPlugin('pendu')
        newfilter = Filter("Pendu_server", [action], condset, 1, 1)
        self.pycc.processor.add_filter(newfilter)
      else:
        triblist = myfilter.conditions.conditions[0].expr[1:-1].split("|")
        if cur_trib.getName() not in triblist:
          triblist.append(cur_trib.getName())
          myfilter.conditions.conditions[0] = Condition('trib', OP_MATCHES, '(%s)' % "|".join(triblist))
      pendu.server_running = 1
      pendu.start_game()
    elif self.res == 0:   # stop server
      if myfilter is not None:
        triblist = [trib for trib in myfilter.conditions.conditions[0].expr[1:-1].split("|") if trib != cur_trib.getName()]
        if len(triblist) > 0:
          myfilter.conditions.conditions[0] = Condition('trib', OP_MATCHES, '(%s)' % "|".join(triblist))
        else:
          self.pycc.processor.remove_filter(pos)
      del self.servers[cur_trib.getName()]
      pendu.server_running = 0
  
  def process(self, bouchot, context, post=None):
    try:
      self.servers[bouchot.name].play(post)
    except KeyError:
      self.error = "Aucun serveur pendu dmarr sur %s" % bouchot.name
      log.info("###", self.error)
