# -*- coding: iso-8859-1 -*-
## ----------------------------------------------------------------------
## PyCoinCoin - Plug-in ThreadView
## ----------------------------------------------------------------------

import logging
log = logging.getLogger('pycc')
import sys
sys.path.append('..')
from coinlib.plugin import *
from cointklib.dialogs import *
from coinlib.utils import apply_styles
from cointklib.basicwidgets import *

import types
_unicode = types.UnicodeType

def build_plugins(pycc):
  return { \
    'threadview': ThreadView(pycc, "ThreadView", "Permet d'afficher une discussion sous forme de thread")
  }

class TVTreeItem:
  
  def __init__(self, post):
    self.id = str(post.id)
    self.label = post.message
    self.post = post

class TVTree(Tree):
  
  def __init__(self, trib, parent, tree, **kwargs):
    self.trib = trib
    Tree.__init__(self, parent, tree, **kwargs)

  def insertItem(self, treeitem):
    post = treeitem.post
    item, tags = text, tags = self.trib.formatPost(post) #, ['id']+self.trib.options.val('disp_post_attrs'))
    itag = "%s" % post.id
    start = self.index(INSERT)
    if type(item) == _unicode:
      text = item
    else:
      try:
        text = item.decode(self.trib.options.val('encoding'))
      except:
        log.warning("ThreadViewTree: Erreur unicode sur %s" % repr(item))
        text = item
    try:
      self.insert(start, text, itag)
    except SystemError:
      # Problme d'incompatibilit entre Py_UNICODE et Tcl_UniChar
      log.debug("ThreadViewTree: Erreur Unicode Python / UniChar Tcl")
      text = item
      self.insert(start, text, itag)
    except TclError:
      log.error("ThreadViewTree: Erreur Tcl en essayant d'crire dans le widget")
      return
    # Ajout des tags
    for tag in tags.keys():
      for range in tags[tag]:
        self.tag_add(tag, "%s+%s chars" % (start, range[0]), "%s+%s chars" % (start, range[1]))    

class TVDialog(CoinTopLevel):
  
  def __init__(self, plugin, trib, tree):
    self.plugin = plugin
    self.trib = trib
    self.tree = tree
    CoinTopLevel.__init__(self, trib.bouchot.pycc.root, "pyCoinCoin - ThreadView", [ 'ExpandAll', 'CollapseAll', 'Fermer' ])
  
  def bind_button(self, button):
    if button == 'ExpandAll':
      return self.expandAll
    else:
      return self.collapseAll
  
  def expandAll(self):
    self.tree.expandAll()
    self.txt.reset()
  
  def collapseAll(self):
    self.tree.collapseAll()
    self.txt.reset()
  
  def body(self, master):
    cnt = Label(master, background="#fcfcda", text=u"Nombre de rponses au post %s: %d" % (self.tree.root.post.dispTimeRef(), self.tree.abslength()-1))
    cnt.pack(pady=3, fill=X)
    Label(master, text="Vue du thread :").pack(anchor=W)
    self.txt = TVTree(self.trib, master, tree=self.tree, hborder=0, hrelief='flat')
    self.txt.pack(fill=BOTH, expand=1)
    self.txt.bind("<Enter>", lambda e, w=self.txt: w.focus_set())
    apply_styles(self.txt, self.trib.bouchot.styles)
    self.txt.select(self.tree.root.id)
    # self.tree.expand(1)
  
  def ok(self, event=None):
    if not self.validate():
      self.initial_focus.focus_set()
      return
    self.withdraw()
    self.update_idletasks()
    self.apply()
    self.parent.canceled = 0
    self.cancel()
  
  def cancel(self, event=None):
    if self.parent is not None:
      self.parent.focus_set()
    self.destroy()
  
  def validate(self):
    return 1 # abstract

class ThreadView(Plugin):
  
  def process(self, bouchot, context=PINNI, post=None):
    # On recre la liste des posts car si on utilisait directement trib.posts, le tableau
    # risque de changer pendant le calcul (vu qu'a priori le thread de polling de la
    # tribune continue de tourner pendant l'excution du plugin)
    self.postlist = [bouchot.trib.posts[id] for id in bouchot.trib.posts.keys()]
    self.postlist.sort()
    tree = self.build_tree(post, bouchot.trib)
    TVDialog(self, bouchot.trib, tree)
  
  def build_tree(self, rootpost, trib):
    rep_found = []
    for post in self.postlist:
      if post <= rootpost: continue
      idlist = []
      refs = [ tag for tag in post.tags.keys() if tag[:3] == "ref" ]
      for ref in refs:
        idlist += trib.getPostsFromRef(ref, post)
      if rootpost.id in idlist:
        rep_found.append(post)
    childs = []
    for post in rep_found:
      childs.append(self.build_tree(post, trib))
    return TreeStruct(TVTreeItem(rootpost), childs)
  